@@ -1,22 +1,22 @@
 /*
- *	  callbacks.h - this file is part of Geany, a fast and lightweight IDE
+ *      callbacks.h - this file is part of Geany, a fast and lightweight IDE
  *
- *	  Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *	  Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
- *	  This program is free software; you can redistribute it and/or modify
- *	  it under the terms of the GNU General Public License as published by
- *	  the Free Software Foundation; either version 2 of the License, or
- *	  (at your option) any later version.
+ *      This program is free software; you can redistribute it and/or modify
+ *      it under the terms of the GNU General Public License as published by
+ *      the Free Software Foundation; either version 2 of the License, or
+ *      (at your option) any later version.
  *
- *	  This program is distributed in the hope that it will be useful,
- *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	  GNU General Public License for more details.
+ *      This program is distributed in the hope that it will be useful,
+ *      but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *      GNU General Public License for more details.
  *
- *    You should have received a copy of the GNU General Public License
- *    along with this program; if not, write to the Free Software
- *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
@@ -327,7 +327,7 @@ on_set_file_readonly1_toggled		  (GtkCheckMenuItem *checkmenuitem,
 
 G_MODULE_EXPORT void
 on_tv_notebook_switch_page			 (GtkNotebook	 *notebook,
-										GtkNotebookPage *page,
+										gpointer	 page,
 										guint			page_num,
 										gpointer		 user_data);
 
@@ -371,7 +371,7 @@ on_toolbutton_run_clicked			  (GtkAction	   *action,
 
 G_MODULE_EXPORT void
 on_notebook1_switch_page_after		 (GtkNotebook	 *notebook,
-										GtkNotebookPage *page,
+										gpointer	 page,
 										guint			page_num,
 										gpointer		 user_data);
 
@@ -508,7 +508,7 @@ on_motion_event						(GtkWidget	   *widget,
 
 G_MODULE_EXPORT void
 on_tv_notebook_switch_page_after	   (GtkNotebook	 *notebook,
-										GtkNotebookPage *page,
+										gpointer	 page,
 										guint			page_num,
 										gpointer		 user_data);
 
@@ -642,10 +642,6 @@ G_MODULE_EXPORT void
 on_reflow_lines_block1_activate		(GtkMenuItem	 *menuitem,
 										gpointer		 user_data);
 
-G_MODULE_EXPORT void
-on_transpose_current_line1_activate	(GtkMenuItem	 *menuitem,
-										gpointer		 user_data);
-
 G_MODULE_EXPORT void
 on_smart_line_indent1_activate		 (GtkMenuItem	 *menuitem,
 										gpointer		 user_data);
