@@ -1,8 +1,8 @@
 /*
  *      filetypes.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: filetypes.c 5964 2011-09-28 10:47:43Z ntrel $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -48,6 +46,7 @@
 
 #include <stdlib.h>
 
+#define GEANY_FILETYPE_SEARCH_LINES 2 /* lines of file to search for filetype */
 
 GPtrArray *filetypes_array = NULL;	/* Dynamic array of filetype pointers */
 
@@ -119,6 +118,14 @@ static void init_builtin_filetypes(void)
 	ft->mime_type = g_strdup("text/x-c++src");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
+#define OBJECTIVEC
+	ft = filetypes[GEANY_FILETYPES_OBJECTIVEC];
+	ft->lang = 42;
+	ft->name = g_strdup("Objective-C");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->mime_type = g_strdup("text/x-objc");
+	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
+
 #define CS
 	ft = filetypes[GEANY_FILETYPES_CS];
 	ft->lang = 25;
@@ -489,6 +496,20 @@ static void init_builtin_filetypes(void)
 	ft->name = g_strdup("Forth");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
+
+#define ASCIIDOC
+	ft = filetypes[GEANY_FILETYPES_ASCIIDOC];
+	ft->lang = 43;
+	ft->name = g_strdup("Asciidoc");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
+
+#define ABAQUS
+	ft = filetypes[GEANY_FILETYPES_ABAQUS];
+	ft->lang = 44;
+	ft->name = g_strdup("Abaqus");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 }
 
 
@@ -637,7 +658,7 @@ void filetypes_init_types()
 		filetype_add(filetypes[ft_id]);
 	}
 	init_custom_filetypes(app->datadir);
-	f = utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, NULL);
+	f = g_build_filename(app->configdir, GEANY_FILEDEFS_SUBDIR, NULL);
 	init_custom_filetypes(f);
 	g_free(f);
 
@@ -655,12 +676,12 @@ static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 
 	g_return_if_fail(NZV(doc->real_path));
 
-	f = utils_build_path(app->configdir, "filetype_extensions.conf", NULL);
+	f = g_build_filename(app->configdir, "filetype_extensions.conf", NULL);
 	if (utils_str_equal(doc->real_path, f))
 		filetypes_reload_extensions();
 
 	g_free(f);
-	f = utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL);
+	f = g_build_filename(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL);
 	if (utils_str_equal(doc->real_path, f))
 	{
 		guint i;
@@ -680,9 +701,9 @@ static void setup_config_file_menus(void)
 {
 	gchar *f;
 
-	f = utils_build_path(app->configdir, "filetype_extensions.conf", NULL);
+	f = g_build_filename(app->configdir, "filetype_extensions.conf", NULL);
 	ui_add_config_file_menu_item(f, NULL, NULL);
-	setptr(f, utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL));
+	SETPTR(f, g_build_filename(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL));
 	ui_add_config_file_menu_item(f, NULL, NULL);
 	g_free(f);
 
@@ -799,13 +820,13 @@ static GeanyFiletype *check_builtin_filenames(const gchar *utf8_filename)
 #else
 	lfn = g_strdup(utf8_filename);
 #endif
-	setptr(lfn, utils_get_locale_from_utf8(lfn));
+	SETPTR(lfn, utils_get_locale_from_utf8(lfn));
 
-	path = utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.", NULL);
+	path = g_build_filename(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.", NULL);
 	if (g_str_has_prefix(lfn, path))
 		found = TRUE;
 
-	setptr(path, utils_build_path(app->datadir, "filetypes.", NULL));
+	SETPTR(path, g_build_filename(app->datadir, "filetypes.", NULL));
 	if (g_str_has_prefix(lfn, path))
 		found = TRUE;
 
@@ -830,7 +851,7 @@ GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename)
 	base_filename = g_path_get_basename(utf8_filename);
 #ifdef G_OS_WIN32
 	/* use lower case basename */
-	setptr(base_filename, g_utf8_strdown(base_filename, -1));
+	SETPTR(base_filename, g_utf8_strdown(base_filename, -1));
 #endif
 
 	ft = filetypes_find(match_basename, base_filename);
@@ -883,47 +904,44 @@ static GeanyFiletype *find_shebang(const gchar *utf8_filename, const gchar *line
 
 	if (strlen(line) > 2 && line[0] == '#' && line[1] == '!')
 	{
+		static const struct {
+			const gchar *name;
+			filetype_id filetype;
+		} intepreter_map[] = {
+			{ "sh",		GEANY_FILETYPES_SH },
+			{ "bash",	GEANY_FILETYPES_SH },
+			{ "dash",	GEANY_FILETYPES_SH },
+			{ "perl",	GEANY_FILETYPES_PERL },
+			{ "python",	GEANY_FILETYPES_PYTHON },
+			{ "php",	GEANY_FILETYPES_PHP },
+			{ "ruby",	GEANY_FILETYPES_RUBY },
+			{ "tcl",	GEANY_FILETYPES_TCL },
+			{ "make",	GEANY_FILETYPES_MAKE },
+			{ "zsh",	GEANY_FILETYPES_SH },
+			{ "ksh",	GEANY_FILETYPES_SH },
+			{ "csh",	GEANY_FILETYPES_SH },
+			{ "ash",	GEANY_FILETYPES_SH },
+			{ "dmd",	GEANY_FILETYPES_D },
+			{ "wish",	GEANY_FILETYPES_TCL }
+		};
 		gchar *tmp = g_path_get_basename(line + 2);
 		gchar *basename_interpreter = tmp;
+		guint i;
 
-		if (strncmp(tmp, "env ", 4) == 0 && strlen(tmp) > 4)
+		if (g_str_has_prefix(tmp, "env "))
 		{	/* skip "env" and read the following interpreter */
 			basename_interpreter += 4;
 		}
 
-		if (strncmp(basename_interpreter, "sh", 2) == 0)
-			ft = filetypes[GEANY_FILETYPES_SH];
-		else if (strncmp(basename_interpreter, "bash", 4) == 0)
-			ft = filetypes[GEANY_FILETYPES_SH];
-		else if (strncmp(basename_interpreter, "perl", 4) == 0)
-			ft = filetypes[GEANY_FILETYPES_PERL];
-		else if (strncmp(basename_interpreter, "python", 6) == 0)
-			ft = filetypes[GEANY_FILETYPES_PYTHON];
-		else if (strncmp(basename_interpreter, "php", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_PHP];
-		else if (strncmp(basename_interpreter, "ruby", 4) == 0)
-			ft = filetypes[GEANY_FILETYPES_RUBY];
-		else if (strncmp(basename_interpreter, "tcl", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_TCL];
-		else if (strncmp(basename_interpreter, "make", 4) == 0)
-			ft = filetypes[GEANY_FILETYPES_MAKE];
-		else if (strncmp(basename_interpreter, "zsh", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_SH];
-		else if (strncmp(basename_interpreter, "ksh", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_SH];
-		else if (strncmp(basename_interpreter, "csh", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_SH];
-		else if (strncmp(basename_interpreter, "ash", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_SH];
-		else if (strncmp(basename_interpreter, "dmd", 3) == 0)
-			ft = filetypes[GEANY_FILETYPES_D];
-		else if (strncmp(basename_interpreter, "wish", 4) == 0)
-			ft = filetypes[GEANY_FILETYPES_TCL];
-
+		for (i = 0; ! ft && i < G_N_ELEMENTS(intepreter_map); i++)
+		{
+			if (g_str_has_prefix(basename_interpreter, intepreter_map[i].name))
+				ft = filetypes[intepreter_map[i].filetype];
+		}
 		g_free(tmp);
 	}
 	/* detect HTML files */
-	if (strncmp(line, "<!DOCTYPE html", 14) == 0 || strncmp(line, "<html", 5) == 0)
+	if (g_str_has_prefix(line, "<!DOCTYPE html") || g_str_has_prefix(line, "<html"))
 	{
 		/* PHP, Perl and Python files might also start with <html, so detect them based on filename
 		 * extension and use the detected filetype, else assume HTML */
@@ -934,7 +952,7 @@ static GeanyFiletype *find_shebang(const gchar *utf8_filename, const gchar *line
 		}
 	}
 	/* detect XML files */
-	else if (utf8_filename && strncmp(line, "<?xml", 5) == 0)
+	else if (utf8_filename && g_str_has_prefix(line, "<?xml"))
 	{
 		/* HTML and DocBook files might also start with <?xml, so detect them based on filename
 		 * extension and use the detected filetype, else assume XML */
@@ -946,24 +964,56 @@ static GeanyFiletype *find_shebang(const gchar *utf8_filename, const gchar *line
 			ft = filetypes[GEANY_FILETYPES_XML];
 		}
 	}
-	else if (strncmp(line, "<?php", 5) == 0)
+	else if (g_str_has_prefix(line, "<?php"))
 	{
 		ft = filetypes[GEANY_FILETYPES_PHP];
 	}
-
 	return ft;
 }
 
 
-/* Detect the filetype checking for a shebang, then filename extension. */
+/* Detect the filetype checking for a shebang, then filename extension.
+ * @lines: an strv of the lines to scan (must containing at least one line) */
 static GeanyFiletype *filetypes_detect_from_file_internal(const gchar *utf8_filename,
-														  const gchar *line)
+														  gchar **lines)
 {
-	GeanyFiletype *ft;
+	GeanyFiletype	*ft;
+	gint			 i;
+	GRegex			*ft_regex;
+	GMatchInfo		*match;
+	GError			*regex_error = NULL;
 
 	/* try to find a shebang and if found use it prior to the filename extension
 	 * also checks for <?xml */
-	ft = find_shebang(utf8_filename, line);
+	ft = find_shebang(utf8_filename, lines[0]);
+	if (ft != NULL)
+		return ft;
+
+	/* try to extract the filetype using a regex capture */
+	ft_regex = g_regex_new(file_prefs.extract_filetype_regex,
+			G_REGEX_RAW | G_REGEX_MULTILINE, 0, &regex_error);
+	if (ft_regex != NULL)
+	{
+		for (i = 0; ft == NULL && lines[i] != NULL; i++)
+		{
+			if (g_regex_match(ft_regex, lines[i], 0, &match))
+			{
+				gchar *capture = g_match_info_fetch(match, 1);
+				if (capture != NULL)
+				{
+					ft = filetypes_lookup_by_name(capture);
+					g_free(capture);
+				}
+			}
+			g_match_info_free(match);
+		}
+		g_regex_unref(ft_regex);
+	}
+	else if (regex_error != NULL)
+	{
+		geany_debug("Filetype extract regex ignored: %s", regex_error->message);
+		g_error_free(regex_error);
+	}
 	if (ft != NULL)
 		return ft;
 
@@ -977,15 +1027,23 @@ static GeanyFiletype *filetypes_detect_from_file_internal(const gchar *utf8_file
 /* Detect the filetype for the document, checking for a shebang, then filename extension. */
 GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc)
 {
-	GeanyFiletype *ft;
-	gchar *line;
+	GeanyFiletype 	*ft;
+	gchar 			*lines[GEANY_FILETYPE_SEARCH_LINES + 1];
+	gint			 i;
 
 	if (doc == NULL)
 		return filetypes[GEANY_FILETYPES_NONE];
 
-	line = sci_get_line(doc->editor->sci, 0);
-	ft = filetypes_detect_from_file_internal(doc->file_name, line);
-	g_free(line);
+	for (i = 0; i < GEANY_FILETYPE_SEARCH_LINES; ++i)
+	{
+		lines[i] = sci_get_line(doc->editor->sci, i);
+	}
+	lines[i] = NULL;
+	ft = filetypes_detect_from_file_internal(doc->file_name, lines);
+	for (i = 0; i < GEANY_FILETYPE_SEARCH_LINES; ++i)
+	{
+		g_free(lines[i]);
+	}
 	return ft;
 }
 
@@ -1003,7 +1061,8 @@ GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc)
 GeanyFiletype *filetypes_detect_from_file(const gchar *utf8_filename)
 {
 	gchar line[1024];
-	FILE *f;
+	gchar *lines[2];
+	FILE  *f;
 	gchar *locale_name = utils_get_locale_from_utf8(utf8_filename);
 
 	f = g_fopen(locale_name, "r");
@@ -1013,7 +1072,9 @@ GeanyFiletype *filetypes_detect_from_file(const gchar *utf8_filename)
 		if (fgets(line, sizeof(line), f) != NULL)
 		{
 			fclose(f);
-			return filetypes_detect_from_file_internal(utf8_filename, line);
+			lines[0] = line;
+			lines[1] = NULL;
+			return filetypes_detect_from_file_internal(utf8_filename, lines);
 		}
 		fclose(f);
 	}
@@ -1059,18 +1120,6 @@ static void create_radio_menu_item(GtkWidget *menu, GeanyFiletype *ftype)
 }
 
 
-static void set_error_regex(GeanyFiletype *ft, gchar *string)
-{
-	setptr(ft->error_regex_string, string);
-
-	if (ft->priv->error_regex_compiled)
-		regfree(&ft->priv->error_regex);
-
-	ft->priv->error_regex_compiled = FALSE;
-	/* regex will be compiled when needed */
-}
-
-
 static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 {
 	GeanyFiletype *ft = data;
@@ -1088,11 +1137,16 @@ static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	g_free(ft->filecmds);
 	g_free(ft->ftdefcmds);
 	g_free(ft->execcmds);
-	set_error_regex(ft, NULL);
+	g_free(ft->error_regex_string);
 	if (ft->icon)
 		g_object_unref(ft->icon);
-
 	g_strfreev(ft->pattern);
+
+	if (ft->priv->error_regex)
+		g_regex_unref(ft->priv->error_regex);
+	g_slist_foreach(ft->priv->tag_files, (GFunc) g_free, NULL);
+	g_slist_free(ft->priv->tag_files);
+
 	g_free(ft->priv);
 	g_free(ft);
 }
@@ -1131,7 +1185,7 @@ static void load_indent_settings(GeanyFiletype *ft, GKeyFile *config, GKeyFile *
 }
 
 
-static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
+static void load_settings(guint ft_id, GKeyFile *config, GKeyFile *configh)
 {
 	GeanyFiletype *ft = filetypes[ft_id];
 	gchar *result;
@@ -1140,31 +1194,31 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	result = utils_get_setting(string, configh, config, "settings", "extension", NULL);
 	if (result != NULL)
 	{
-		setptr(filetypes[ft_id]->extension, result);
+		SETPTR(filetypes[ft_id]->extension, result);
 	}
 
 	/* read comment notes */
 	result = utils_get_setting(string, configh, config, "settings", "comment_open", NULL);
 	if (result != NULL)
 	{
-		setptr(filetypes[ft_id]->comment_open, result);
+		SETPTR(filetypes[ft_id]->comment_open, result);
 	}
 
 	result = utils_get_setting(string, configh, config, "settings", "comment_close", NULL);
 	if (result != NULL)
 	{
-		setptr(filetypes[ft_id]->comment_close, result);
+		SETPTR(filetypes[ft_id]->comment_close, result);
 	}
 
 	result = utils_get_setting(string, configh, config, "settings", "comment_single", NULL);
 	if (result != NULL)
 	{
-		setptr(filetypes[ft_id]->comment_single, result);
+		SETPTR(filetypes[ft_id]->comment_single, result);
 	}
 	/* import correctly filetypes that use old-style single comments */
 	else if (! NZV(filetypes[ft_id]->comment_close))
 	{
-		setptr(filetypes[ft_id]->comment_single, filetypes[ft_id]->comment_open);
+		SETPTR(filetypes[ft_id]->comment_single, filetypes[ft_id]->comment_open);
 		filetypes[ft_id]->comment_open = NULL;
 	}
 
@@ -1175,7 +1229,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	result = utils_get_setting(string, configh, config, "settings", "context_action_cmd", NULL);
 	if (result != NULL)
 	{
-		setptr(filetypes[ft_id]->context_action_cmd, result);
+		SETPTR(filetypes[ft_id]->context_action_cmd, result);
 	}
 
 	result = utils_get_setting(string, configh, config, "settings", "tag_parser", NULL);
@@ -1230,16 +1284,18 @@ static void add_keys(GKeyFile *dest, const gchar *group, GKeyFile *src)
 static gchar *filetypes_get_filename(GeanyFiletype *ft, gboolean user)
 {
 	gchar *ext = filetypes_get_conf_extension(ft);
-	gchar *f;
+	gchar *base_name = g_strconcat("filetypes.", ext, NULL);
+	gchar *file_name;
 
 	if (user)
-		f = utils_make_filename(app->configdir,
-			GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S, "filetypes.", ext, NULL);
+		file_name = g_build_filename(app->configdir, GEANY_FILEDEFS_SUBDIR, base_name, NULL);
 	else
-		f = utils_make_filename(app->datadir, "filetypes.", ext, NULL);
+		file_name = g_build_filename(app->datadir, base_name, NULL);
 
 	g_free(ext);
-	return f;
+	g_free(base_name);
+
+	return file_name;
 }
 
 
@@ -1321,13 +1377,13 @@ static void load_system_keyfile(GKeyFile *key_file, const gchar *file, GKeyFileF
 /* Load the configuration file for the associated filetype id.
  * This should only be called when the filetype is needed, to save loading
  * 20+ configuration files all at once. */
-void filetypes_load_config(gint ft_id, gboolean reload)
+void filetypes_load_config(guint ft_id, gboolean reload)
 {
 	GKeyFile *config, *config_home;
 	GeanyFiletypePrivate *pft;
 	GeanyFiletype *ft;
 
-	g_return_if_fail(ft_id >= 0 && ft_id < (gint) filetypes_array->len);
+	g_return_if_fail(ft_id < filetypes_array->len);
 
 	ft = filetypes[ft_id];
 	pft = ft->priv;
@@ -1350,7 +1406,7 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 		f = filetypes_get_filename(ft, FALSE);
 		load_system_keyfile(config, f, G_KEY_FILE_KEEP_COMMENTS, ft);
 
-		setptr(f, filetypes_get_filename(ft, TRUE));
+		SETPTR(f, filetypes_get_filename(ft, TRUE));
 		g_key_file_load_from_file(config_home, f, G_KEY_FILE_KEEP_COMMENTS, NULL);
 		g_free(f);
 	}
@@ -1382,6 +1438,8 @@ static gchar *filetypes_get_conf_extension(const GeanyFiletype *ft)
 		case GEANY_FILETYPES_NONE: result = g_strdup("common"); break;
 		/* name is Matlab/Octave */
 		case GEANY_FILETYPES_MATLAB: result = g_strdup("matlab"); break;
+		/* name is Objective-C, and we don't want the hyphen */
+		case GEANY_FILETYPES_OBJECTIVEC: result = g_strdup("objectivec"); break;
 		default:
 			result = g_ascii_strdown(ft->name, -1);
 			break;
@@ -1480,27 +1538,20 @@ GeanyFiletype *filetypes_lookup_by_name(const gchar *name)
 }
 
 
-static gchar *get_regex_match_string(const gchar *message, regmatch_t *pmatch, gint match_idx)
+static void compile_regex(GeanyFiletype *ft, gchar *regstr)
 {
-	return g_strndup(&message[pmatch[match_idx].rm_so],
-		pmatch[match_idx].rm_eo - pmatch[match_idx].rm_so);
-}
+	GError *error = NULL;
+	GRegex *regex = g_regex_new(regstr, 0, 0, &error);
 
-
-static void compile_regex(GeanyFiletype *ft, regex_t *regex, gchar *regstr)
-{
-	gint retval = regcomp(regex, regstr, REG_EXTENDED);
-
-	ft->priv->error_regex_compiled = (retval == 0);	/* prevent recompilation */
-
-	if (G_UNLIKELY(retval != 0))
+	if (!regex)
 	{
-		gchar buf[256];
-		regerror(retval, regex, buf, sizeof buf);
 		ui_set_statusbar(TRUE, _("Bad regex for filetype %s: %s"),
-			filetypes_get_display_name(ft), buf);
+			filetypes_get_display_name(ft), error->message);
+		g_error_free(error);
 	}
-	/* regex will be freed in set_error_regex(). */
+	if (ft->priv->error_regex)
+		g_regex_unref(ft->priv->error_regex);
+	ft->priv->error_regex = regex;
 }
 
 
@@ -1510,8 +1561,7 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 	gchar *regstr;
 	gchar **tmp;
 	GeanyDocument *doc;
-	regex_t *regex;
-	regmatch_t pmatch[3];
+	GMatchInfo *minfo;
 
 	if (ft == NULL)
 	{
@@ -1523,7 +1573,6 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 	if (tmp == NULL)
 		return FALSE;
 	regstr = *tmp;
-	regex = &ft->priv->error_regex;
 
 	*filename = NULL;
 	*line = -1;
@@ -1531,24 +1580,26 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 	if (G_UNLIKELY(! NZV(regstr)))
 		return FALSE;
 
-	if (!ft->priv->error_regex_compiled || regstr != ft->priv->last_string)
+	if (!ft->priv->error_regex || regstr != ft->priv->last_error_pattern)
 	{
-		compile_regex(ft, regex, regstr);
-		ft->priv->last_string = regstr;
+		compile_regex(ft, regstr);
+		ft->priv->last_error_pattern = regstr;
 	}
-	if (!ft->priv->error_regex_compiled)	/* regex error */
+	if (!ft->priv->error_regex)
 		return FALSE;
 
-	if (regexec(regex, message, G_N_ELEMENTS(pmatch), pmatch, 0) != 0)
+	if (!g_regex_match(ft->priv->error_regex, message, 0, &minfo))
+	{
+		g_match_info_free(minfo);
 		return FALSE;
-
-	if (pmatch[0].rm_so != -1 && pmatch[1].rm_so != -1 && pmatch[2].rm_so != -1)
+	}
+	if (g_match_info_get_match_count(minfo) >= 3)
 	{
 		gchar *first, *second, *end;
 		glong l;
 
-		first = get_regex_match_string(message, pmatch, 1);
-		second = get_regex_match_string(message, pmatch, 2);
+		first = g_match_info_fetch(minfo, 1);
+		second = g_match_info_fetch(minfo, 2);
 		l = strtol(first, &end, 10);
 		if (*end == '\0')	/* first is purely decimals */
 		{
@@ -1572,6 +1623,7 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 			}
 		}
 	}
+	g_match_info_free(minfo);
 	return *filename != NULL;
 }
 
@@ -1582,7 +1634,7 @@ static void convert_filetype_extensions_to_lower_case(gchar **patterns, gsize le
 	guint i;
 	for (i = 0; i < len; i++)
 	{
-		setptr(patterns[i], g_ascii_strdown(patterns[i], -1));
+		SETPTR(patterns[i], g_ascii_strdown(patterns[i], -1));
 	}
 }
 #endif
@@ -1629,7 +1681,7 @@ static void read_group(GKeyFile *config, const gchar *group_name, gint group_id)
 			if (ft->priv->custom &&
 				(group_id == GEANY_FILETYPE_GROUP_COMPILED || group_id == GEANY_FILETYPE_GROUP_SCRIPT))
 			{
-				setptr(ft->title, NULL);
+				SETPTR(ft->title, NULL);
 				filetype_make_title(ft, TITLE_SOURCE_FILE);
 			}
 		}
@@ -1652,10 +1704,8 @@ static void read_groups(GKeyFile *config)
 
 static void read_filetype_config(void)
 {
-	gchar *sysconfigfile = g_strconcat(app->datadir, G_DIR_SEPARATOR_S,
-		"filetype_extensions.conf", NULL);
-	gchar *userconfigfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S,
-		"filetype_extensions.conf", NULL);
+	gchar *sysconfigfile = g_build_filename(app->datadir, "filetype_extensions.conf", NULL);
+	gchar *userconfigfile = g_build_filename(app->configdir, "filetype_extensions.conf", NULL);
 	GKeyFile *sysconfig = g_key_file_new();
 	GKeyFile *userconfig = g_key_file_new();
 
@@ -1739,3 +1789,39 @@ const gchar *filetypes_get_display_name(GeanyFiletype *ft)
 	return ft->id == GEANY_FILETYPES_NONE ? _("None") : ft->name;
 }
 
+
+/* gets comment_open/comment_close/comment_single strings from the filetype
+ * @param single_first: whether single comment is preferred if both available
+ * returns true if at least comment_open is set, false otherwise */
+gboolean filetype_get_comment_open_close(const GeanyFiletype *ft, gboolean single_first,
+		const gchar **co, const gchar **cc)
+{
+	g_return_val_if_fail(ft != NULL, FALSE);
+	g_return_val_if_fail(co != NULL, FALSE);
+	g_return_val_if_fail(cc != NULL, FALSE);
+
+	if (single_first)
+	{
+		*co = ft->comment_single;
+		if (NZV(*co))
+			*cc = NULL;
+		else
+		{
+			*co = ft->comment_open;
+			*cc = ft->comment_close;
+		}
+	}
+	else
+	{
+		*co = ft->comment_open;
+		if (NZV(*co))
+			*cc = ft->comment_close;
+		else
+		{
+			*co = ft->comment_single;
+			*cc = NULL;
+		}
+	}
+
+	return NZV(*co);
+}
