@@ -1,24 +1,22 @@
 /*
  *      geanymenubuttonaction.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either vergeany 2 of the License, or
- *      (at your option) any later vergeany.
+ *      the Free Software Foundation; either version 2 of the License, or
+ *      (at your option) any later version.
  *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: geanymenubuttonaction.c 5845 2011-06-13 21:01:42Z colombanw $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /* GtkAction subclass to provide a GtkMenuToolButton in a toolbar.
@@ -196,7 +194,12 @@ static void menu_items_changed_cb(GtkContainer *container, GtkWidget *widget, Ge
 
 	priv = GEANY_MENU_BUTTON_ACTION_GET_PRIVATE(action);
 	if (priv->menu != NULL)
-		enable = (g_list_length(gtk_container_get_children(GTK_CONTAINER(priv->menu))) > 0);
+	{
+		GList *children = gtk_container_get_children(GTK_CONTAINER(priv->menu));
+
+		enable = (g_list_length(children) > 0);
+		g_list_free(children);
+	}
 	else
 		enable = FALSE;
 
