@@ -1,29 +1,27 @@
 /*
  *      geanymenubuttonaction.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either vergeany 2 of the License, or
- *      (at your option) any later vergeany.
+ *      the Free Software Foundation; either version 2 of the License, or
+ *      (at your option) any later version.
  *
  *      This program is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: geanymenubuttonaction.h 5528 2011-01-19 19:39:09Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
-#ifndef __GEANY_MENU_BUTTON_ACTION_H__
-#define __GEANY_MENU_BUTTON_ACTION_H__
+#ifndef GEANY_MENU_BUTTON_ACTION_H
+#define GEANY_MENU_BUTTON_ACTION_H
 
 G_BEGIN_DECLS
 
@@ -65,4 +63,4 @@ void		geany_menu_button_action_set_menu	(GeanyMenubuttonAction *action, GtkWidge
 
 G_END_DECLS
 
-#endif /* __GEANY_MENU_BUTTON_ACTION_H__ */
+#endif /* GEANY_MENU_BUTTON_ACTION_H */
