@@ -15,9 +15,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -629,16 +629,38 @@ static void styleset_common_init(GKeyFile *config, GKeyFile *config_home)
 }
 
 
+static void set_character_classes(ScintillaObject *sci, guint ft_id)
+{
+	const gchar *word = (ft_id == GEANY_FILETYPES_NONE ?
+		common_style_set.wordchars : style_sets[ft_id].wordchars);
+	gchar *whitespace;
+	guint i, j;
+
+	SSM(sci, SCI_SETWORDCHARS, 0, (sptr_t) word);
+
+	/* setting wordchars resets character classes, so we have to set whitespaces after
+	 * wordchars, but we want wordchars to have precenence over whitepace chars */
+	whitespace = g_malloc0(strlen(whitespace_chars) + 1);
+	for (i = 0, j = 0; whitespace_chars[i] != 0; i++)
+	{
+		if (! strchr(word, whitespace_chars[i]))
+			whitespace[j++] = whitespace_chars[i];
+	}
+	whitespace[j] = 0;
+
+	SSM(sci, SCI_SETWHITESPACECHARS, 0, (sptr_t) whitespace);
+
+	g_free(whitespace);
+}
+
+
 static void styleset_common(ScintillaObject *sci, guint ft_id)
 {
 	GeanyLexerStyle *style;
 
 	SSM(sci, SCI_STYLECLEARALL, 0, 0);
 
-	SSM(sci, SCI_SETWORDCHARS, 0, (sptr_t) (ft_id == GEANY_FILETYPES_NONE ?
-		common_style_set.wordchars : style_sets[ft_id].wordchars));
-	/* have to set whitespace after setting wordchars */
-	SSM(sci, SCI_SETWHITESPACECHARS, 0, (sptr_t) whitespace_chars);
+	set_character_classes(sci, ft_id);
 
 	/* caret colour, style and width */
 	SSM(sci, SCI_SETCARETFORE, invert(common_style_set.styling[GCS_CARET].foreground), 0);
@@ -670,7 +692,7 @@ static void styleset_common(ScintillaObject *sci, guint ft_id)
 	SSM(sci, SCI_SETWRAPINDENTMODE, common_style_set.styling[GCS_LINE_WRAP_INDENT].background, 0);
 
 	/* Error indicator */
-	SSM(sci, SCI_INDICSETSTYLE, GEANY_INDICATOR_ERROR, INDIC_SQUIGGLE);
+	SSM(sci, SCI_INDICSETSTYLE, GEANY_INDICATOR_ERROR, INDIC_SQUIGGLEPIXMAP);
 	SSM(sci, SCI_INDICSETFORE, GEANY_INDICATOR_ERROR, invert(rotate_rgb(0xff0000)));
 
 	/* Search indicator, used for 'Mark' matches */
@@ -1017,6 +1039,7 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 
 	switch (lexer_id)
 	{
+		init_styleset_case(ABAQUS);
 		init_styleset_case(ADA);
 		init_styleset_case(ASM);
 		init_styleset_case(BASIC);
@@ -1096,6 +1119,7 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 
 	switch (lexer_id)
 	{
+		styleset_case(ABAQUS);
 		styleset_case(ADA);
 		styleset_case(ASM);
 		styleset_case(BASIC);
@@ -1377,7 +1401,7 @@ void highlighting_show_color_scheme_dialog(void)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(swin), tree);
-	gtk_container_add(GTK_CONTAINER(vbox), swin);
+	gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
 	g_signal_connect(dialog, "response", G_CALLBACK(on_color_scheme_dialog_response), &dialog);
 	gtk_widget_show_all(dialog);
 }
@@ -1402,7 +1426,8 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_C_STRINGEOL ||
 				style == SCE_C_STRINGRAW ||
 				style == SCE_C_VERBATIM ||
-				style == SCE_C_TRIPLEVERBATIM);
+				style == SCE_C_TRIPLEVERBATIM ||
+				style == SCE_C_HASHQUOTEDSTRING);
 
 		case SCLEX_PASCAL:
 			return (style == SCE_PAS_CHARACTER ||
@@ -1446,6 +1471,14 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_PL_XLAT
 				/* we don't include any STRING_*_VAR for autocompletion */);
 
+		case SCLEX_PO:
+			return (style == SCE_PO_MSGCTXT_TEXT ||
+				style == SCE_PO_MSGCTXT_TEXT_EOL ||
+				style == SCE_PO_MSGID_TEXT ||
+				style == SCE_PO_MSGID_TEXT_EOL ||
+				style == SCE_PO_MSGSTR_TEXT ||
+				style == SCE_PO_MSGSTR_TEXT_EOL);
+
 		case SCLEX_R:
 			return (style == SCE_R_STRING);
 
@@ -1532,6 +1565,9 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_ADA_STRING ||
 				style == SCE_ADA_CHARACTEREOL ||
 				style == SCE_ADA_STRINGEOL);
+
+		case SCLEX_ABAQUS:
+			return (style == SCE_ABAQUS_STRING);
 	}
 	return FALSE;
 }
@@ -1553,6 +1589,7 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 			return (style == SCE_C_COMMENT ||
 				style == SCE_C_COMMENTLINE ||
 				style == SCE_C_COMMENTDOC ||
+				style == SCE_C_PREPROCESSORCOMMENT ||
 				style == SCE_C_COMMENTLINEDOC ||
 				style == SCE_C_COMMENTDOCKEYWORD ||
 				style == SCE_C_COMMENTDOCKEYWORDERROR);
@@ -1586,7 +1623,8 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 			return (style == SCE_PROPS_COMMENT);
 
 		case SCLEX_PO:
-			return (style == SCE_PO_COMMENT);
+			return (style == SCE_PO_COMMENT ||
+				style == SCE_PO_PROGRAMMER_COMMENT);
 
 		case SCLEX_LATEX:
 			return (style == SCE_L_COMMENT ||
@@ -1667,6 +1705,10 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 			return (style == SCE_ADA_COMMENTLINE ||
 				style == SCE_NSIS_COMMENTBOX);
 
+		case SCLEX_ABAQUS:
+			return (style == SCE_ABAQUS_COMMENT ||
+				 style == SCE_ABAQUS_COMMENTBLOCK);
+
 		case SCLEX_ASM:
 			return (style == SCE_ASM_COMMENT ||
 				style == SCE_ASM_COMMENTBLOCK ||
