@@ -15,9 +15,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
@@ -69,6 +69,31 @@ typedef struct
 #define HL_N_ENTRIES(array) ((array != NULL) ? G_N_ELEMENTS(array) : 0)
 
 
+/* Abaqus */
+#define highlighting_lexer_ABAQUS			SCLEX_ABAQUS
+static const HLStyle highlighting_styles_ABAQUS[] =
+{
+	{ SCE_ABAQUS_DEFAULT,		"default",		FALSE },
+	{ SCE_ABAQUS_COMMENT,		"comment",		FALSE },
+	{ SCE_ABAQUS_NUMBER,		"number",		FALSE },
+	{ SCE_ABAQUS_STRING,		"string",		FALSE },
+	{ SCE_ABAQUS_OPERATOR,		"operator",		FALSE },
+	{ SCE_ABAQUS_PROCESSOR,		"processor",		FALSE },
+	{ SCE_ABAQUS_STARCOMMAND,	"starcommand",		FALSE },
+	{ SCE_ABAQUS_ARGUMENT,		"argument",		FALSE }
+};
+static const HLKeyword highlighting_keywords_ABAQUS[] =
+{
+	{ 0, "processors", FALSE },
+	{ 1, "commands", FALSE },
+	{ 2, "slashommands", FALSE },
+	{ 3, "starcommands", FALSE },
+	{ 4, "arguments", FALSE },
+	{ 5, "functions", FALSE }
+};
+#define highlighting_properties_ABAQUS		EMPTY_PROPERTIES
+
+
 /* Ada */
 #define highlighting_lexer_ADA			SCLEX_ADA
 static const HLStyle highlighting_styles_ADA[] =
@@ -180,10 +205,12 @@ static const HLStyle highlighting_styles_C[] =
 	{ SCE_C_COMMENT,				"comment",					FALSE },
 	{ SCE_C_COMMENTLINE,			"commentline",				FALSE },
 	{ SCE_C_COMMENTDOC,				"commentdoc",				FALSE },
+	{ SCE_C_PREPROCESSORCOMMENT,	"preprocessorcomment",		FALSE },
 	{ SCE_C_NUMBER,					"number",					FALSE },
 	{ SCE_C_WORD,					"word",						FALSE },
 	{ SCE_C_WORD2,					"word2",					FALSE },
 	{ SCE_C_STRING,					"string",					FALSE },
+	{ SCE_C_STRINGRAW,				"stringraw",				FALSE },
 	{ SCE_C_CHARACTER,				"character",				FALSE },
 	{ SCE_C_UUID,					"uuid",						FALSE },
 	{ SCE_C_PREPROCESSOR,			"preprocessor",				FALSE },
@@ -194,6 +221,7 @@ static const HLStyle highlighting_styles_C[] =
 	/* triple verbatims use the same style */
 	{ SCE_C_TRIPLEVERBATIM,			"verbatim",					FALSE },
 	{ SCE_C_REGEX,					"regex",					FALSE },
+	{ SCE_C_HASHQUOTEDSTRING,		"hashquotedstring",			FALSE },
 	{ SCE_C_COMMENTLINEDOC,			"commentlinedoc",			FALSE },
 	{ SCE_C_COMMENTDOCKEYWORD,		"commentdockeyword",		FALSE },
 	{ SCE_C_COMMENTDOCKEYWORDERROR,	"commentdockeyworderror",	FALSE },
@@ -291,6 +319,7 @@ static const HLStyle highlighting_styles_CSS[] =
 	{ SCE_CSS_VALUE,					"value",					FALSE },
 	{ SCE_CSS_ID,						"id",						FALSE },
 	{ SCE_CSS_IDENTIFIER2,				"identifier2",				FALSE },
+	{ SCE_CSS_VARIABLE,					"variable",					FALSE },
 	{ SCE_CSS_IMPORTANT,				"important",				FALSE },
 	{ SCE_CSS_DIRECTIVE,				"directive",				FALSE },
 	{ SCE_CSS_IDENTIFIER3,				"identifier3",				FALSE },
@@ -543,7 +572,12 @@ static const HLStyle highlighting_styles_FORTH[] =
 };
 static const HLKeyword highlighting_keywords_FORTH[] =
 {
-	{ 0, "primary",	FALSE }
+	{ 0, "primary",		FALSE },
+	{ 1, "keyword",		FALSE },
+	{ 2, "defword",		FALSE },
+	{ 3, "preword1",	FALSE },
+	{ 4, "preword2",	FALSE },
+	{ 5, "string",		FALSE }
 };
 #define highlighting_properties_FORTH	EMPTY_PROPERTIES
 
@@ -1071,15 +1105,22 @@ static const HLKeyword highlighting_keywords_PERL[] =
 #define highlighting_lexer_PO		SCLEX_PO
 static const HLStyle highlighting_styles_PO[] =
 {
-	{ SCE_PO_DEFAULT,		"default",			FALSE },
-	{ SCE_PO_COMMENT,		"comment",			FALSE },
-	{ SCE_PO_MSGID,			"msgid",			FALSE },
-	{ SCE_PO_MSGID_TEXT,	"msgid_text",		FALSE },
-	{ SCE_PO_MSGSTR,		"msgstr",			FALSE },
-	{ SCE_PO_MSGSTR_TEXT,	"msgstr_text",		FALSE },
-	{ SCE_PO_MSGCTXT,		"msgctxt",			FALSE },
-	{ SCE_PO_MSGCTXT_TEXT,	"msgctxt_text",		FALSE },
-	{ SCE_PO_FUZZY,			"fuzzy",			FALSE }
+	{ SCE_PO_DEFAULT,				"default",				FALSE },
+	{ SCE_PO_COMMENT,				"comment",				FALSE },
+	{ SCE_PO_PROGRAMMER_COMMENT,	"programmer_comment",	FALSE },
+	{ SCE_PO_REFERENCE,				"reference",			FALSE },
+	{ SCE_PO_FLAGS,					"flags",				FALSE },
+	{ SCE_PO_FUZZY,					"fuzzy",				FALSE },
+	{ SCE_PO_MSGID,					"msgid",				FALSE },
+	{ SCE_PO_MSGID_TEXT,			"msgid_text",			FALSE },
+	{ SCE_PO_MSGID_TEXT_EOL,		"msgid_text_eol",		FALSE },
+	{ SCE_PO_MSGSTR,				"msgstr",				FALSE },
+	{ SCE_PO_MSGSTR_TEXT,			"msgstr_text",			FALSE },
+	{ SCE_PO_MSGSTR_TEXT_EOL,		"msgstr_text_eol",		FALSE },
+	{ SCE_PO_MSGCTXT,				"msgctxt",				FALSE },
+	{ SCE_PO_MSGCTXT_TEXT,			"msgctxt_text",			FALSE },
+	{ SCE_PO_MSGCTXT_TEXT_EOL,		"msgctxt_text_eol",		FALSE },
+	{ SCE_PO_ERROR,					"error",				FALSE }
 };
 #define highlighting_keywords_PO	EMPTY_KEYWORDS
 #define highlighting_properties_PO	EMPTY_PROPERTIES
