@@ -1,8 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: msgwindow.c 5887 2011-08-13 20:44:39Z colombanw $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -549,7 +547,7 @@ static GtkWidget *create_message_popup_menu(gint type)
 
 	message_popup_menu = gtk_menu_new();
 
-	clear = gtk_image_menu_item_new_from_stock("gtk-clear", NULL);
+	clear = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR, NULL);
 	gtk_widget_show(clear);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), clear);
 	g_signal_connect(clear, "activate",
@@ -558,7 +556,7 @@ static GtkWidget *create_message_popup_menu(gint type)
 	copy = gtk_image_menu_item_new_with_mnemonic(_("C_opy"));
 	gtk_widget_show(copy);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
-	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy), image);
 	g_signal_connect(copy, "activate",
@@ -567,7 +565,7 @@ static GtkWidget *create_message_popup_menu(gint type)
 	copy_all = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
 	gtk_widget_show(copy_all);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy_all);
-	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy_all), image);
 	g_signal_connect(copy_all, "activate",
@@ -643,9 +641,9 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboole
 		if (cur_dir)
 		{
 			/* we let the user know we couldn't find the parsed filename from the message window */
-			setptr(cur_dir, utils_get_locale_from_utf8(cur_dir));
+			SETPTR(cur_dir, utils_get_locale_from_utf8(cur_dir));
 			name = g_path_get_basename(filename);
-			setptr(name, g_build_path(G_DIR_SEPARATOR_S, cur_dir, name, NULL));
+			SETPTR(name, g_build_path(G_DIR_SEPARATOR_S, cur_dir, name, NULL));
 			g_free(cur_dir);
 
 			if (g_file_test(name, G_FILE_TEST_EXISTS))
@@ -746,8 +744,7 @@ static void make_absolute(gchar **filename, const gchar *dir)
 
 	/* add directory */
 	if (! utils_is_absolute_path(*filename))
-		setptr(*filename, g_strconcat(dir, G_DIR_SEPARATOR_S,
-			*filename + skip_dot_slash, NULL));
+		SETPTR(*filename, g_build_filename(dir, *filename + skip_dot_slash, NULL));
 }
 
 
@@ -846,11 +843,25 @@ static void parse_compiler_error_line(const gchar *string,
 		case GEANY_FILETYPES_PYTHON:
 		{
 			/* File "HyperArch.py", line 37, in ?
-			 * (file "clrdial.tcl" line 12) */
-			data.pattern = " \"";
-			data.min_fields = 6;
-			data.line_idx = 5;
-			data.file_idx = 2;
+			 * (file "clrdial.tcl" line 12)
+			 * */
+			if (strstr(string, " line ") != NULL)
+			{
+				/* Tcl and old Python format (<= Python 2.5) */
+				data.pattern = " \"";
+				data.min_fields = 6;
+				data.line_idx = 5;
+				data.file_idx = 2;
+			}
+			else
+			{
+				/* SyntaxError: ('invalid syntax', ('sender.py', 149, 20, ' ...'))
+				 * (used since Python 2.6) */
+				data.pattern = ",'";
+				data.min_fields = 8;
+				data.line_idx = 6;
+				data.file_idx = 4;
+			}
 			break;
 		}
 		case GEANY_FILETYPES_BASIC:
@@ -1037,7 +1048,7 @@ static void msgwin_parse_generic_line(const gchar *string, gchar **filename, gin
 		/* if we aren't sure we got a supposedly correct filename, check it */
 		if (incertain && ! g_file_test(*filename, G_FILE_TEST_EXISTS))
 		{
-			setptr(*filename, NULL);
+			SETPTR(*filename, NULL);
 			*line = -1;
 		}
 	}
@@ -1211,5 +1222,3 @@ void msgwin_clear_tab(gint tabnum)
 		return;
 	gtk_list_store_clear(store);
 }
-
-
