@@ -14,9 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -185,11 +185,11 @@ static void kb_tree_view_change_button_clicked_cb(GtkWidget *button, gpointer da
 					_("Press the combination of the keys you want to use for \"%s\"."), name);
 			label = gtk_label_new(str);
 			gtk_misc_set_padding(GTK_MISC(label), 5, 10);
-			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
+			gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), label);
 
 			dialog_label = gtk_label_new("");
 			gtk_misc_set_padding(GTK_MISC(dialog_label), 5, 10);
-			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), dialog_label);
+			gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), dialog_label);
 
 			g_signal_connect(dialog, "key-press-event",
 								G_CALLBACK(kb_grab_key_dialog_key_press_cb), NULL);
@@ -744,6 +744,9 @@ static void prefs_init_dialog(void)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "color_back");
 		gtk_color_button_set_color(GTK_COLOR_BUTTON(widget), vc->colour_back);
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_image");
+		gtk_entry_set_text(GTK_ENTRY(widget), vc->image);
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_scrollback");
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), vc->scrollback_lines);
 
@@ -857,6 +860,9 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		if (interface_prefs.sidebar_pos != old_sidebar_pos)
 			ui_swap_sidebar_pos();
 
+		widget = ui_lookup_widget(main_widgets.window, "vpaned1");
+		gtk_orientable_set_orientation(GTK_ORIENTABLE(widget), interface_prefs.msgwin_orientation);
+
 		/* General settings */
 		/* startup */
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_load_session");
@@ -1205,6 +1211,10 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			gtk_spin_button_update(GTK_SPIN_BUTTON(widget));
 			vc->scrollback_lines = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 
+			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_image");
+			g_free(vc->image);
+			vc->image = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
+
 			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_shell");
 			g_free(vc->shell);
 			vc->shell = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
@@ -1632,14 +1642,10 @@ void prefs_show_dialog(void)
 		GtkWidget *label;
 		guint i;
 		gchar *encoding_string;
-		GdkPixbuf *pb;
 
 		ui_widgets.prefs_dialog = create_prefs_dialog();
 		gtk_widget_set_name(ui_widgets.prefs_dialog, "GeanyPrefsDialog");
 		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.prefs_dialog), GTK_WINDOW(main_widgets.window));
-		pb = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO);
-		gtk_window_set_icon(GTK_WINDOW(ui_widgets.prefs_dialog), pb);
-		g_object_unref(pb);	/* free our reference */
 
 		/* init the file encoding combo boxes */
 		encoding_list = ui_builder_get_object("encoding_list");
