@@ -1,8 +1,8 @@
 /*
  *      sciwrappers.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: sciwrappers.c 5652 2011-03-29 23:52:03Z colombanw $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /** @file sciwrappers.h
@@ -49,7 +47,7 @@ void sci_set_line_numbers(ScintillaObject *sci, gboolean set, gint extra_width)
 	if (set)
 	{
 		gchar tmp_str[15];
-		gint len = SSM(sci, SCI_GETLINECOUNT, 0, 0);
+		gint len = (gint) SSM(sci, SCI_GETLINECOUNT, 0, 0);
 		gint width;
 
 		g_snprintf(tmp_str, 15, "_%d", len);
@@ -71,6 +69,8 @@ void sci_set_line_numbers(ScintillaObject *sci, gboolean set, gint extra_width)
 
 void sci_set_mark_long_lines(ScintillaObject *sci, gint type, gint column, const gchar *colour)
 {
+	glong colour_val = utils_strtod(colour, NULL, TRUE); /* Scintilla uses a "long" value */
+
 	if (column == 0)
 		type = 2;
 	switch (type)
@@ -91,8 +91,8 @@ void sci_set_mark_long_lines(ScintillaObject *sci, gint type, gint column, const
 			return;
 		}
 	}
-	SSM(sci, SCI_SETEDGECOLUMN, column, 0);
-	SSM(sci, SCI_SETEDGECOLOUR, utils_strtod(colour, NULL, TRUE), 0);
+	SSM(sci, SCI_SETEDGECOLUMN, (uptr_t) column, 0);
+	SSM(sci, SCI_SETEDGECOLOUR, (uptr_t) colour_val, 0);
 }
 
 
@@ -131,7 +131,7 @@ void sci_set_folding_margin_visible(ScintillaObject *sci, gboolean set)
 /* end of lines */
 void sci_set_visible_eols(ScintillaObject *sci, gboolean set)
 {
-	SSM(sci, SCI_SETVIEWEOL, set, 0);
+	SSM(sci, SCI_SETVIEWEOL, set != FALSE, 0);
 }
 
 
@@ -155,19 +155,19 @@ void sci_set_lines_wrapped(ScintillaObject *sci, gboolean set)
 
 gint sci_get_eol_mode(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETEOLMODE, 0, 0);
+	return (gint) SSM(sci, SCI_GETEOLMODE, 0, 0);
 }
 
 
 void sci_set_eol_mode(ScintillaObject *sci, gint eolmode)
 {
-	SSM(sci, SCI_SETEOLMODE, eolmode, 0);
+	SSM(sci, SCI_SETEOLMODE, (uptr_t) eolmode, 0);
 }
 
 
 void sci_convert_eols(ScintillaObject *sci, gint eolmode)
 {
-	SSM(sci, SCI_CONVERTEOLS, eolmode, 0);
+	SSM(sci, SCI_CONVERTEOLS, (uptr_t) eolmode, 0);
 }
 
 
@@ -193,13 +193,13 @@ void sci_set_text(ScintillaObject *sci, const gchar *text)
 
 gboolean sci_can_undo(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_CANUNDO, 0, 0);
+	return SSM(sci, SCI_CANUNDO, 0, 0) != FALSE;
 }
 
 
 gboolean sci_can_redo(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_CANREDO, 0, 0);
+	return SSM(sci, SCI_CANREDO, 0, 0) != FALSE;
 }
 
 
@@ -237,7 +237,7 @@ void sci_end_undo_action(ScintillaObject *sci)
 
 void sci_set_undo_collection(ScintillaObject *sci, gboolean set)
 {
-	SSM(sci, SCI_SETUNDOCOLLECTION, set, 0);
+	SSM(sci, SCI_SETUNDOCOLLECTION, set != FALSE, 0);
 }
 
 
@@ -273,7 +273,7 @@ void sci_zoom_off(ScintillaObject *sci)
 
 gint sci_get_zoom(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETZOOM, 0, 0);
+	return (gint) SSM(sci, SCI_GETZOOM, 0, 0);
 }
 
 
@@ -283,7 +283,7 @@ gint sci_get_zoom(ScintillaObject *sci)
  * @param marker Marker number. */
 void sci_set_marker_at_line(ScintillaObject *sci, gint line_number, gint marker)
 {
-	SSM(sci, SCI_MARKERADD, line_number, marker);
+	SSM(sci, SCI_MARKERADD, (uptr_t) line_number, marker);
 }
 
 
@@ -293,7 +293,7 @@ void sci_set_marker_at_line(ScintillaObject *sci, gint line_number, gint marker)
  * @param marker Marker number. */
 void sci_delete_marker_at_line(ScintillaObject *sci, gint line_number, gint marker)
 {
-	SSM(sci, SCI_MARKERDELETE, line_number, marker);
+	SSM(sci, SCI_MARKERDELETE, (uptr_t) line_number, marker);
 }
 
 
@@ -306,7 +306,7 @@ gboolean sci_is_marker_set_at_line(ScintillaObject *sci, gint line, gint marker)
 {
 	gint state;
 
-	state = SSM(sci, SCI_MARKERGET, line, 0);
+	state = (gint) SSM(sci, SCI_MARKERGET, (uptr_t) line, 0);
 	return (state & (1 << marker));
 }
 
@@ -329,9 +329,9 @@ gint sci_marker_next(ScintillaObject *sci, gint line, gint marker_mask, gboolean
 {
 	gint marker_line;
 
-	marker_line = SSM(sci, SCI_MARKERNEXT, line, marker_mask);
+	marker_line = (gint) SSM(sci, SCI_MARKERNEXT, (uptr_t) line, marker_mask);
 	if (wrap && marker_line == -1)
-		marker_line = SSM(sci, SCI_MARKERNEXT, 0, marker_mask);
+		marker_line = (gint) SSM(sci, SCI_MARKERNEXT, 0, marker_mask);
 	return marker_line;
 }
 
@@ -343,13 +343,13 @@ gint sci_marker_previous(ScintillaObject *sci, gint line, gint marker_mask, gboo
 {
 	gint marker_line;
 
-	marker_line = SSM(sci, SCI_MARKERPREVIOUS, line, marker_mask);
+	marker_line = (gint) SSM(sci, SCI_MARKERPREVIOUS, (uptr_t) line, marker_mask);
 	if (wrap && marker_line == -1)
 	{
 		gint len = sci_get_length(sci);
 		gint last_line = sci_get_line_from_position(sci, len - 1);
 
-		marker_line = SSM(sci, SCI_MARKERPREVIOUS, last_line, marker_mask);
+		marker_line = (gint) SSM(sci, SCI_MARKERPREVIOUS, (uptr_t) last_line, marker_mask);
 	}
 	return marker_line;
 }
@@ -361,7 +361,7 @@ gint sci_marker_previous(ScintillaObject *sci, gint line, gint marker_mask, gboo
  * @return The line. */
 gint sci_get_line_from_position(ScintillaObject *sci, gint position)
 {
-	return SSM(sci, SCI_LINEFROMPOSITION, position, 0);
+	return (gint) SSM(sci, SCI_LINEFROMPOSITION, (uptr_t) position, 0);
 }
 
 
@@ -371,7 +371,7 @@ gint sci_get_line_from_position(ScintillaObject *sci, gint position)
  * @return The column. */
 gint sci_get_col_from_position(ScintillaObject *sci, gint position)
 {
-	return SSM(sci, SCI_GETCOLUMN, position, 0);
+	return (gint) SSM(sci, SCI_GETCOLUMN, (uptr_t) position, 0);
 }
 
 
@@ -381,7 +381,7 @@ gint sci_get_col_from_position(ScintillaObject *sci, gint position)
  * @return Position. */
 gint sci_get_position_from_line(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_POSITIONFROMLINE, line, 0);
+	return (gint) SSM(sci, SCI_POSITIONFROMLINE, (uptr_t) line, 0);
 }
 
 
@@ -390,7 +390,7 @@ gint sci_get_position_from_line(ScintillaObject *sci, gint line)
  * @return Position. */
 gint sci_get_current_position(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETCURRENTPOS, 0, 0);
+	return (gint) SSM(sci, SCI_GETCURRENTPOS, 0, 0);
 }
 
 
@@ -401,11 +401,11 @@ gint sci_get_current_position(ScintillaObject *sci)
 void sci_set_current_position(ScintillaObject *sci, gint position, gboolean scroll_to_caret)
 {
 	if (scroll_to_caret)
-		SSM(sci, SCI_GOTOPOS, position, 0);
+		SSM(sci, SCI_GOTOPOS, (uptr_t) position, 0);
 	else
 	{
-		SSM(sci, SCI_SETCURRENTPOS, position, 0);
-		SSM(sci, SCI_SETANCHOR, position, 0); /* to avoid creation of a selection */
+		SSM(sci, SCI_SETCURRENTPOS, (uptr_t) position, 0);
+		SSM(sci, SCI_SETANCHOR, (uptr_t) position, 0); /* to avoid creation of a selection */
 	}
 	SSM(sci, SCI_CHOOSECARETX, 0, 0);
 }
@@ -425,7 +425,7 @@ void sci_set_current_line(ScintillaObject *sci, gint line)
  * @return The line count. */
 gint sci_get_line_count(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETLINECOUNT, 0, 0);
+	return (gint) SSM(sci, SCI_GETLINECOUNT, 0, 0);
 }
 
 
@@ -434,7 +434,7 @@ gint sci_get_line_count(ScintillaObject *sci)
  * @param position Position. */
 void sci_set_selection_start(ScintillaObject *sci, gint position)
 {
-	SSM(sci, SCI_SETSELECTIONSTART, position, 0);
+	SSM(sci, SCI_SETSELECTIONSTART, (uptr_t) position, 0);
 }
 
 
@@ -443,13 +443,13 @@ void sci_set_selection_start(ScintillaObject *sci, gint position)
  * @param position Position. */
 void sci_set_selection_end(ScintillaObject *sci, gint position)
 {
-	SSM(sci, SCI_SETSELECTIONEND, position, 0);
+	SSM(sci, SCI_SETSELECTIONEND, (uptr_t) position, 0);
 }
 
 
 void sci_set_selection(ScintillaObject *sci, gint anchorPos, gint currentPos)
 {
-	SSM(sci, SCI_SETSEL, anchorPos, currentPos);
+	SSM(sci, SCI_SETSEL, (uptr_t) anchorPos, currentPos);
 }
 
 
@@ -459,7 +459,7 @@ void sci_set_selection(ScintillaObject *sci, gint anchorPos, gint currentPos)
  * @return The position at the end of the line. */
 gint sci_get_line_end_position(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETLINEENDPOSITION, line, 0);
+	return (gint) SSM(sci, SCI_GETLINEENDPOSITION, (uptr_t) line, 0);
 }
 
 
@@ -492,7 +492,7 @@ void sci_clear(ScintillaObject *sci)
  * @return Position. */
 gint sci_get_selection_start(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETSELECTIONSTART, 0, 0);
+	return (gint) SSM(sci, SCI_GETSELECTIONSTART, 0, 0);
 }
 
 
@@ -501,7 +501,7 @@ gint sci_get_selection_start(ScintillaObject *sci)
  * @return Position. */
 gint sci_get_selection_end(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETSELECTIONEND, 0, 0);
+	return (gint) SSM(sci, SCI_GETSELECTIONEND, 0, 0);
 }
 
 
@@ -519,13 +519,28 @@ void sci_replace_sel(ScintillaObject *sci, const gchar *text)
  * @return Length. */
 gint sci_get_length(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETLENGTH, 0, 0);
+	return (gint) SSM(sci, SCI_GETLENGTH, 0, 0);
 }
 
 
+/** Gets the currently used lexer
+ * @param sci Scintilla widget.
+ * @returns The lexer ID
+ */
 gint sci_get_lexer(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETLEXER, 0, 0);
+	return (gint) SSM(sci, SCI_GETLEXER, 0, 0);
+}
+
+
+void sci_set_lexer(ScintillaObject *sci, guint lexer_id)
+{
+	gint old = sci_get_lexer(sci);
+
+	SSM(sci, SCI_SETLEXER, lexer_id, 0);
+
+	if (old != (gint)lexer_id)
+		SSM(sci, SCI_CLEARDOCUMENTSTYLE, 0, 0);
 }
 
 
@@ -535,19 +550,19 @@ gint sci_get_lexer(ScintillaObject *sci)
  * @return Length. */
 gint sci_get_line_length(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_LINELENGTH, line, 0);
+	return (gint) SSM(sci, SCI_LINELENGTH, (uptr_t) line, 0);
 }
 
 
 /* safe way to read Scintilla string into new memory.
  * works with any string buffer messages that follow the Windows message convention. */
-gchar *sci_get_string(ScintillaObject *sci, gint msg, gulong wParam)
+gchar *sci_get_string(ScintillaObject *sci, guint msg, gulong wParam)
 {
-	gint size = SSM(sci, msg, wParam, 0) + 1;
-	gchar *str = g_malloc(size);
+	gint size = (gint) SSM(sci, msg, wParam, 0);
+	gchar *str = g_malloc(size + 1);
 
-	SSM(sci, msg, wParam, (sptr_t)str);
-	str[size - 1] = '\0';	/* ensure termination, needed for SCI_GETLINE */
+	SSM(sci, msg, wParam, (sptr_t) str);
+	str[size] = '\0';	/* ensure termination, needed for SCI_GETLINE */
 	return str;
 }
 
@@ -558,7 +573,7 @@ gchar *sci_get_string(ScintillaObject *sci, gint msg, gulong wParam)
  * @return A @c NULL-terminated copy of the line text. */
 gchar *sci_get_line(ScintillaObject *sci, gint line_num)
 {
-	return sci_get_string(sci, SCI_GETLINE, line_num);
+	return sci_get_string(sci, SCI_GETLINE, (gulong) line_num);
 }
 
 
@@ -571,22 +586,28 @@ gchar *sci_get_line(ScintillaObject *sci, gint line_num)
  * @param text Text buffer; must be allocated @a len + 1 bytes for null-termination. */
 void sci_get_text(ScintillaObject *sci, gint len, gchar *text)
 {
-	SSM(sci, SCI_GETTEXT, len, (sptr_t) text);
+	SSM(sci, SCI_GETTEXT, (uptr_t) len, (sptr_t) text);
 }
 
 
-/** Gets all text inside a given text length.
+/** Allocates and fills a buffer with text from the start of the document.
  * @param sci Scintilla widget.
- * @param len Length of the text to retrieve from the start of the document,
- *            usually sci_get_length() + 1.
+ * @param buffer_len Buffer length to allocate, including the terminating
+ * null char, e.g. sci_get_length() + 1. Alternatively use @c -1 to get all
+ * text (since Geany 1.23).
  * @return A copy of the text. Should be freed when no longer needed.
  *
- * @since 0.17
+ * @since 1.23 (0.17)
  */
-gchar *sci_get_contents(ScintillaObject *sci, gint len)
+gchar *sci_get_contents(ScintillaObject *sci, gint buffer_len)
 {
-	gchar *text = g_malloc(len);
-	SSM(sci, SCI_GETTEXT, len, (sptr_t) text);
+	gchar *text;
+
+	if (buffer_len < 0)
+		buffer_len = sci_get_length(sci) + 1;
+
+	text = g_malloc(buffer_len);
+	SSM(sci, SCI_GETTEXT, (uptr_t) buffer_len, (sptr_t) text);
 	return text;
 }
 
@@ -622,14 +643,14 @@ gchar *sci_get_selection_contents(ScintillaObject *sci)
  * @return Length. */
 gint sci_get_selected_text_length(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETSELTEXT, 0, 0);
+	return (gint) SSM(sci, SCI_GETSELTEXT, 0, 0);
 }
 
 
 gint sci_get_position_from_xy(ScintillaObject *sci, gint x, gint y, gboolean nearby)
 {
 	/* for nearby return -1 if there is no character near to the x,y point. */
-	return SSM(sci, (nearby) ? SCI_POSITIONFROMPOINTCLOSE : SCI_POSITIONFROMPOINT, x, y);
+	return (gint) SSM(sci, (nearby) ? SCI_POSITIONFROMPOINTCLOSE : SCI_POSITIONFROMPOINT, (uptr_t) x, y);
 }
 
 
@@ -639,7 +660,7 @@ gint sci_get_position_from_xy(ScintillaObject *sci, gint x, gint y, gboolean nea
  * @return Whether @a line will be drawn on the screen. */
 gboolean sci_get_line_is_visible(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETLINEVISIBLE, line, 0);
+	return SSM(sci, SCI_GETLINEVISIBLE, (uptr_t) line, 0) != FALSE;
 }
 
 
@@ -648,38 +669,38 @@ gboolean sci_get_line_is_visible(ScintillaObject *sci, gint line)
  * @param line Line number. */
 void sci_ensure_line_is_visible(ScintillaObject *sci, gint line)
 {
-	SSM(sci, SCI_ENSUREVISIBLE, line, 0);
+	SSM(sci, SCI_ENSUREVISIBLE, (uptr_t) line, 0);
 }
 
 
 gint sci_get_fold_level(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETFOLDLEVEL, line, 0);
+	return (gint) SSM(sci, SCI_GETFOLDLEVEL, (uptr_t) line, 0);
 }
 
 
 /* Get the line number of the fold point before start_line, or -1 if there isn't one */
 gint sci_get_fold_parent(ScintillaObject *sci, gint start_line)
 {
-	return SSM(sci, SCI_GETFOLDPARENT, start_line, 0);
+	return (gint) SSM(sci, SCI_GETFOLDPARENT, (uptr_t) start_line, 0);
 }
 
 
 void sci_toggle_fold(ScintillaObject *sci, gint line)
 {
-	SSM(sci, SCI_TOGGLEFOLD, line, 1);
+	SSM(sci, SCI_TOGGLEFOLD, (uptr_t) line, 0);
 }
 
 
 gboolean sci_get_fold_expanded(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETFOLDEXPANDED, line, 0);
+	return SSM(sci, SCI_GETFOLDEXPANDED, (uptr_t) line, 0) != FALSE;
 }
 
 
 void sci_colourise(ScintillaObject *sci, gint start, gint end)
 {
-	SSM(sci, SCI_COLOURISE, start, end);
+	SSM(sci, SCI_COLOURISE, (uptr_t) start, end);
 }
 
 
@@ -691,13 +712,13 @@ void sci_clear_all(ScintillaObject *sci)
 
 gint sci_get_end_styled(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETENDSTYLED, 0, 0);
+	return (gint) SSM(sci, SCI_GETENDSTYLED, 0, 0);
 }
 
 
 void sci_set_tab_width(ScintillaObject *sci, gint width)
 {
-	SSM(sci, SCI_SETTABWIDTH, width, 0);
+	SSM(sci, SCI_SETTABWIDTH, (uptr_t) width, 0);
 }
 
 
@@ -709,7 +730,7 @@ void sci_set_tab_width(ScintillaObject *sci, gint width)
  **/
 gint sci_get_tab_width(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETTABWIDTH, 0, 0);
+	return (gint) SSM(sci, SCI_GETTABWIDTH, 0, 0);
 }
 
 
@@ -719,7 +740,7 @@ gint sci_get_tab_width(ScintillaObject *sci)
  * @return Char. */
 gchar sci_get_char_at(ScintillaObject *sci, gint pos)
 {
-	return (gchar) SSM(sci, SCI_GETCHARAT, pos, 0);
+	return (gchar) SSM(sci, SCI_GETCHARAT, (uptr_t) pos, 0);
 }
 
 
@@ -731,13 +752,13 @@ void sci_set_savepoint(ScintillaObject *sci)
 
 void sci_set_indentation_guides(ScintillaObject *sci, gint mode)
 {
-	SSM(sci, SCI_SETINDENTATIONGUIDES, mode, 0);
+	SSM(sci, SCI_SETINDENTATIONGUIDES, (uptr_t) mode, 0);
 }
 
 
 void sci_use_popup(ScintillaObject *sci, gboolean enable)
 {
-	SSM(sci, SCI_USEPOPUP, enable, 0);
+	SSM(sci, SCI_USEPOPUP, enable != FALSE, 0);
 }
 
 
@@ -758,8 +779,8 @@ gboolean sci_has_selection(ScintillaObject *sci)
 
 void sci_goto_pos(ScintillaObject *sci, gint pos, gboolean unfold)
 {
-	if (unfold) SSM(sci, SCI_ENSUREVISIBLE, SSM(sci, SCI_LINEFROMPOSITION, pos, 0), 0);
-	SSM(sci, SCI_GOTOPOS, pos, 0);
+	if (unfold) SSM(sci, SCI_ENSUREVISIBLE, (uptr_t) SSM(sci, SCI_LINEFROMPOSITION, (uptr_t) pos, 0), 0);
+	SSM(sci, SCI_GOTOPOS, (uptr_t) pos, 0);
 }
 
 
@@ -775,7 +796,7 @@ void sci_set_anchor(ScintillaObject *sci, gint pos)
 	if (pos < 0)
 		pos = sci_get_current_position(sci);
 
-	SSM(sci, SCI_SETANCHOR, pos, 0);
+	SSM(sci, SCI_SETANCHOR, (uptr_t) pos, 0);
 }
 
 
@@ -795,19 +816,21 @@ void sci_scroll_lines(ScintillaObject *sci, gint lines)
 
 void sci_scroll_columns(ScintillaObject *sci, gint columns)
 {
-	SSM(sci, SCI_LINESCROLL, columns, 0);
+	SSM(sci, SCI_LINESCROLL, (uptr_t) columns, 0);
 }
 
 
 gint sci_search_next(ScintillaObject *sci, gint flags, const gchar *text)
 {
-	return SSM(sci, SCI_SEARCHNEXT, flags, (sptr_t) text);
+	/* FIXME: SCI_SEACHNEXT() actually returns long */
+	return (gint) SSM(sci, SCI_SEARCHNEXT, (uptr_t) flags, (sptr_t) text);
 }
 
 
 gint sci_search_prev(ScintillaObject *sci, gint flags, const gchar *text)
 {
-	return SSM(sci, SCI_SEARCHPREV, flags, (sptr_t) text);
+	/* FIXME: SCI_SEACHPREV() actually returns long */
+	return (gint) SSM(sci, SCI_SEARCHPREV, (uptr_t) flags, (sptr_t) text);
 }
 
 
@@ -826,7 +849,7 @@ gint sci_search_prev(ScintillaObject *sci, gint flags, const gchar *text)
  */
 gint sci_find_text(ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf)
 {
-	return SSM(sci, SCI_FINDTEXT, flags, (long) ttf);
+	return (gint) SSM(sci, SCI_FINDTEXT, (uptr_t) flags, (sptr_t) ttf);
 }
 
 
@@ -837,8 +860,8 @@ gint sci_find_text(ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf)
  * @param size The font size. */
 void sci_set_font(ScintillaObject *sci, gint style, const gchar *font, gint size)
 {
-	SSM(sci, SCI_STYLESETFONT, style, (sptr_t) font);
-	SSM(sci, SCI_STYLESETSIZE, style, size);
+	SSM(sci, SCI_STYLESETFONT, (uptr_t) style, (sptr_t) font);
+	SSM(sci, SCI_STYLESETSIZE, (uptr_t) style, size);
 }
 
 
@@ -851,14 +874,14 @@ void sci_set_font(ScintillaObject *sci, gint style, const gchar *font, gint size
  */
 void sci_goto_line(ScintillaObject *sci, gint line, gboolean unfold)
 {
-	if (unfold) SSM(sci, SCI_ENSUREVISIBLE, line, 0);
-	SSM(sci, SCI_GOTOLINE, line, 0);
+	if (unfold) SSM(sci, SCI_ENSUREVISIBLE, (uptr_t) line, 0);
+	SSM(sci, SCI_GOTOLINE, (uptr_t) line, 0);
 }
 
 
 void sci_marker_delete_all(ScintillaObject *sci, gint marker)
 {
-	SSM(sci, SCI_MARKERDELETEALL, marker, 0);
+	SSM(sci, SCI_MARKERDELETEALL, (uptr_t) marker, 0);
 }
 
 
@@ -868,26 +891,26 @@ void sci_marker_delete_all(ScintillaObject *sci, gint marker)
  * @return Style ID. */
 gint sci_get_style_at(ScintillaObject *sci, gint position)
 {
-	return SSM(sci, SCI_GETSTYLEAT, position, 0);
+	return (gint) SSM(sci, SCI_GETSTYLEAT, (uptr_t) position, 0);
 }
 
 
 void sci_set_codepage(ScintillaObject *sci, gint cp)
 {
 	g_return_if_fail(cp == 0 || cp == SC_CP_UTF8);
-	SSM(sci, SCI_SETCODEPAGE, cp, 0);
+	SSM(sci, SCI_SETCODEPAGE, (uptr_t) cp, 0);
 }
 
 
 void sci_assign_cmdkey(ScintillaObject *sci, gint key, gint command)
 {
-	SSM(sci, SCI_ASSIGNCMDKEY, key, command);
+	SSM(sci, SCI_ASSIGNCMDKEY, (uptr_t) key, command);
 }
 
 
 void sci_clear_cmdkey(ScintillaObject *sci, gint key)
 {
-	SSM(sci, SCI_CLEARCMDKEY, key, 0);
+	SSM(sci, SCI_CLEARCMDKEY, (uptr_t) key, 0);
 }
 
 
@@ -910,17 +933,20 @@ void sci_get_text_range(ScintillaObject *sci, gint start, gint end, gchar *text)
 
 
 /** Gets text between @a start and @a end.
- *
  * @param sci Scintilla widget.
- * @param start Start.
- * @param end End.
+ * @param start Start position.
+ * @param end End position.
  * @return The text inside the given range. Should be freed when no longer needed.
  *
  * @since 0.17
  */
 gchar *sci_get_contents_range(ScintillaObject *sci, gint start, gint end)
 {
-	gchar *text = g_malloc((end - start) + 1);
+	gchar *text;
+
+	g_return_val_if_fail(start < end, NULL);
+
+	text = g_malloc((gsize) (end - start) + 1);
 	sci_get_text_range(sci, start, end, text);
 	return text;
 }
@@ -944,7 +970,7 @@ void sci_selection_duplicate(ScintillaObject *sci)
  * @param text Text. */
 void sci_insert_text(ScintillaObject *sci, gint pos, const gchar *text)
 {
-	SSM(sci, SCI_INSERTTEXT, pos, (sptr_t) text);
+	SSM(sci, SCI_INSERTTEXT, (uptr_t) pos, (sptr_t) text);
 }
 
 
@@ -956,23 +982,23 @@ void sci_target_from_selection(ScintillaObject *sci)
 
 void sci_set_target_start(ScintillaObject *sci, gint start)
 {
-	SSM(sci, SCI_SETTARGETSTART, start, 0);
+	SSM(sci, SCI_SETTARGETSTART, (uptr_t) start, 0);
 }
 
 
 void sci_set_target_end(ScintillaObject *sci, gint end)
 {
-	SSM(sci, SCI_SETTARGETEND, end, 0);
+	SSM(sci, SCI_SETTARGETEND, (uptr_t) end, 0);
 }
 
 
 gint sci_replace_target(ScintillaObject *sci, const gchar *text, gboolean regex)
 {
-	return SSM(sci, (regex) ? SCI_REPLACETARGETRE : SCI_REPLACETARGET, (uptr_t) -1, (sptr_t) text);
+	return (gint) SSM(sci, (regex) ? SCI_REPLACETARGETRE : SCI_REPLACETARGET, (uptr_t) -1, (sptr_t) text);
 }
 
 
-void sci_set_keywords(ScintillaObject *sci, gint k, const gchar *text)
+void sci_set_keywords(ScintillaObject *sci, guint k, const gchar *text)
 {
 	SSM(sci, SCI_SETKEYWORDS, k, (sptr_t) text);
 }
@@ -980,7 +1006,7 @@ void sci_set_keywords(ScintillaObject *sci, gint k, const gchar *text)
 
 void sci_set_readonly(ScintillaObject *sci, gboolean readonly)
 {
-	SSM(sci, SCI_SETREADONLY, readonly, 0);
+	SSM(sci, SCI_SETREADONLY, readonly != FALSE, 0);
 }
 
 
@@ -1002,7 +1028,7 @@ void sci_send_command(ScintillaObject *sci, gint cmd)
  * @return Line number. */
 gint sci_get_current_line(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_LINEFROMPOSITION, SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
+	return (gint) SSM(sci, SCI_LINEFROMPOSITION, (uptr_t) SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
 }
 
 
@@ -1011,19 +1037,24 @@ gint sci_get_current_line(ScintillaObject *sci)
  * Returns 2 if a whole line is selected including the line break char(s). */
 gint sci_get_lines_selected(ScintillaObject *sci)
 {
-	gint start = SSM(sci, SCI_GETSELECTIONSTART, 0, 0);
-	gint end = SSM(sci, SCI_GETSELECTIONEND, 0, 0);
+	gint start = (gint) SSM(sci, SCI_GETSELECTIONSTART, 0, 0);
+	gint end = (gint) SSM(sci, SCI_GETSELECTIONEND, 0, 0);
+	gint line_start;
+	gint line_end;
 
 	if (start == end)
 		return 0; /* no selection */
 
-	return SSM(sci, SCI_LINEFROMPOSITION, end, 0) - SSM(sci, SCI_LINEFROMPOSITION, start, 0) + 1;
+	line_start = (gint) SSM(sci, SCI_LINEFROMPOSITION, (uptr_t) start, 0);
+	line_end = (gint) SSM(sci, SCI_LINEFROMPOSITION, (uptr_t) end, 0);
+
+	return line_end - line_start + 1;
 }
 
 
 gint sci_get_first_visible_line(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETFIRSTVISIBLELINE, 0, 0);
+	return (gint) SSM(sci, SCI_GETFIRSTVISIBLELINE, 0, 0);
 }
 
 
@@ -1040,13 +1071,13 @@ gint sci_get_first_visible_line(ScintillaObject *sci)
  */
 void sci_indicator_set(ScintillaObject *sci, gint indic)
 {
-	SSM(sci, SCI_SETINDICATORCURRENT, indic, 0);
+	SSM(sci, SCI_SETINDICATORCURRENT, (uptr_t) indic, 0);
 }
 
 
 void sci_indicator_fill(ScintillaObject *sci, gint pos, gint len)
 {
-	SSM(sci, SCI_INDICATORFILLRANGE, pos, len);
+	SSM(sci, SCI_INDICATORFILLRANGE, (uptr_t) pos, len);
 }
 
 
@@ -1064,7 +1095,7 @@ void sci_indicator_fill(ScintillaObject *sci, gint pos, gint len)
  */
 void sci_indicator_clear(ScintillaObject *sci, gint pos, gint len)
 {
-	SSM(sci, SCI_INDICATORCLEARRANGE, pos, len);
+	SSM(sci, SCI_INDICATORCLEARRANGE, (uptr_t) pos, len);
 }
 
 
@@ -1076,13 +1107,13 @@ void sci_select_all(ScintillaObject *sci)
 
 gint sci_get_line_indent_position(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETLINEINDENTPOSITION, line, 0);
+	return (gint) SSM(sci, SCI_GETLINEINDENTPOSITION, (uptr_t) line, 0);
 }
 
 
 void sci_set_autoc_max_height(ScintillaObject *sci, gint val)
 {
-	SSM(sci, SCI_AUTOCSETMAXHEIGHT, val, 0);
+	SSM(sci, SCI_AUTOCSETMAXHEIGHT, (uptr_t) val, 0);
 }
 
 
@@ -1095,37 +1126,37 @@ void sci_set_autoc_max_height(ScintillaObject *sci, gint val)
  **/
 gint sci_find_matching_brace(ScintillaObject *sci, gint pos)
 {
-	return SSM(sci, SCI_BRACEMATCH, pos, 0);
+	return (gint) SSM(sci, SCI_BRACEMATCH, (uptr_t) pos, 0);
 }
 
 
 gint sci_get_overtype(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETOVERTYPE, 0, 0);
+	return (gint) SSM(sci, SCI_GETOVERTYPE, 0, 0);
 }
 
 
 void sci_set_tab_indents(ScintillaObject *sci, gboolean set)
 {
-	SSM(sci, SCI_SETTABINDENTS, set, 0);
+	SSM(sci, SCI_SETTABINDENTS, set != FALSE, 0);
 }
 
 
 void sci_set_use_tabs(ScintillaObject *sci, gboolean set)
 {
-	SSM(sci, SCI_SETUSETABS, set, 0);
+	SSM(sci, SCI_SETUSETABS, set != FALSE, 0);
 }
 
 
 gint sci_get_pos_at_line_sel_start(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETLINESELSTARTPOSITION, line, 0);
+	return (gint) SSM(sci, SCI_GETLINESELSTARTPOSITION, (uptr_t) line, 0);
 }
 
 
 gint sci_get_pos_at_line_sel_end(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETLINESELENDPOSITION, line, 0);
+	return (gint) SSM(sci, SCI_GETLINESELENDPOSITION, (uptr_t) line, 0);
 }
 
 
@@ -1134,7 +1165,7 @@ gint sci_get_pos_at_line_sel_end(ScintillaObject *sci, gint line)
  * @return Selection mode. */
 gint sci_get_selection_mode(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETSELECTIONMODE, 0, 0);
+	return (gint) SSM(sci, SCI_GETSELECTIONMODE, 0, 0);
 }
 
 
@@ -1143,14 +1174,14 @@ gint sci_get_selection_mode(ScintillaObject *sci)
  * @param mode Mode. */
 void sci_set_selection_mode(ScintillaObject *sci, gint mode)
 {
-	SSM(sci, SCI_SETSELECTIONMODE, mode, 0);
+	SSM(sci, SCI_SETSELECTIONMODE, (uptr_t) mode, 0);
 }
 
 
 void sci_set_scrollbar_mode(ScintillaObject *sci, gboolean visible)
 {
-	SSM(sci, SCI_SETHSCROLLBAR, visible, 0);
-	SSM(sci, SCI_SETVSCROLLBAR, visible, 0);
+	SSM(sci, SCI_SETHSCROLLBAR, visible != FALSE, 0);
+	SSM(sci, SCI_SETVSCROLLBAR, visible != FALSE, 0);
 }
 
 
@@ -1163,7 +1194,7 @@ void sci_set_scrollbar_mode(ScintillaObject *sci, gboolean visible)
  */
 void sci_set_line_indentation(ScintillaObject *sci, gint line, gint indent)
 {
-	SSM(sci, SCI_SETLINEINDENTATION, line, indent);
+	SSM(sci, SCI_SETLINEINDENTATION, (uptr_t) line, indent);
 }
 
 
@@ -1176,25 +1207,25 @@ void sci_set_line_indentation(ScintillaObject *sci, gint line, gint indent)
  */
 gint sci_get_line_indentation(ScintillaObject *sci, gint line)
 {
-	return SSM(sci, SCI_GETLINEINDENTATION, line, 0);
+	return (gint) SSM(sci, SCI_GETLINEINDENTATION, (uptr_t) line, 0);
 }
 
 
 void sci_set_caret_policy_x(ScintillaObject *sci, gint policy, gint slop)
 {
-	SSM(sci, SCI_SETXCARETPOLICY, policy, slop);
+	SSM(sci, SCI_SETXCARETPOLICY, (uptr_t) policy, slop);
 }
 
 
 void sci_set_caret_policy_y(ScintillaObject *sci, gint policy, gint slop)
 {
-	SSM(sci, SCI_SETYCARETPOLICY, policy, slop);
+	SSM(sci, SCI_SETYCARETPOLICY, (uptr_t) policy, slop);
 }
 
 
 void sci_set_scroll_stop_at_last_line(ScintillaObject *sci, gboolean set)
 {
-	SSM(sci, SCI_SETENDATLASTLINE, set, 0);
+	SSM(sci, SCI_SETENDATLASTLINE, set != FALSE, 0);
 }
 
 
@@ -1206,19 +1237,19 @@ void sci_cancel(ScintillaObject *sci)
 
 gint sci_get_target_end(ScintillaObject *sci)
 {
-	return SSM(sci, SCI_GETTARGETEND, 0, 0);
+	return (gint) SSM(sci, SCI_GETTARGETEND, 0, 0);
 }
 
 
 gint sci_get_position_after(ScintillaObject *sci, gint start)
 {
-	return SSM(sci, SCI_POSITIONAFTER, start, 0);
+	return (gint) SSM(sci, SCI_POSITIONAFTER, (uptr_t) start, 0);
 }
 
 
 void sci_lines_split(ScintillaObject *sci, gint pixelWidth)
 {
-	SSM(sci, SCI_LINESSPLIT, pixelWidth, 0);
+	SSM(sci, SCI_LINESSPLIT, (uptr_t) pixelWidth, 0);
 }
 
 
@@ -1230,5 +1261,15 @@ void sci_lines_join(ScintillaObject *sci)
 
 gint sci_text_width(ScintillaObject *sci, gint styleNumber, const gchar *text)
 {
-	return SSM(sci, SCI_TEXTWIDTH, styleNumber, (sptr_t) text);
+	return (gint) SSM(sci, SCI_TEXTWIDTH, (uptr_t) styleNumber, (sptr_t) text);
+}
+
+void sci_move_selected_lines_down(ScintillaObject *sci)
+{
+	SSM(sci, SCI_MOVESELECTEDLINESDOWN, 0, 0);
+}
+
+void sci_move_selected_lines_up(ScintillaObject *sci)
+{
+	SSM(sci, SCI_MOVESELECTEDLINESUP, 0, 0);
 }
