@@ -1,8 +1,8 @@
 /*
  *      sciwrappers.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: sciwrappers.h 5528 2011-01-19 19:39:09Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef GEANY_SCIWRAPPERS_H
@@ -28,7 +26,7 @@
 #include "ScintillaWidget.h"
 
 
-gchar*				sci_get_string				(ScintillaObject *sci, gint msg, gulong wParam);
+gchar*				sci_get_string				(ScintillaObject *sci, guint msg, gulong wParam);
 
 void 				sci_set_line_numbers		(ScintillaObject *sci,  gboolean set, gint extra_width);
 void				sci_set_mark_long_lines		(ScintillaObject *sci,	gint type, gint column, const gchar *color);
@@ -86,7 +84,7 @@ void				sci_set_selection			(ScintillaObject *sci, gint anchorPos, gint currentP
 
 gint				sci_get_length				(ScintillaObject *sci);
 void				sci_get_text				(ScintillaObject *sci, gint len, gchar *text);
-gchar*				sci_get_contents			(ScintillaObject *sci, gint len);
+gchar*				sci_get_contents			(ScintillaObject *sci, gint buffer_len);
 void				sci_get_selected_text		(ScintillaObject *sci, gchar *text);
 gint				sci_get_selected_text_length(ScintillaObject *sci);
 gchar*				sci_get_selection_contents	(ScintillaObject *sci);
@@ -146,8 +144,9 @@ void				sci_set_target_end			(ScintillaObject *sci, gint end);
 gint				sci_get_target_end			(ScintillaObject *sci);
 gint				sci_replace_target			(ScintillaObject *sci, const gchar *text, gboolean regex);
 
-void				sci_set_keywords			(ScintillaObject *sci, gint k, const gchar *text);
+void				sci_set_keywords			(ScintillaObject *sci, guint k, const gchar *text);
 gint				sci_get_lexer				(ScintillaObject *sci);
+void				sci_set_lexer				(ScintillaObject *sci, guint lexer_id);
 void				sci_set_readonly			(ScintillaObject *sci, gboolean readonly);
 gint				sci_get_zoom				(ScintillaObject *sci);
 void				sci_send_command			(ScintillaObject *sci, gint cmd);
@@ -185,4 +184,7 @@ void				sci_lines_split				(ScintillaObject *sci, gint pixelWidth);
 void				sci_lines_join				(ScintillaObject *sci);
 gint				sci_text_width				(ScintillaObject *sci, gint styleNumber, const gchar *text);
 
+void				sci_move_selected_lines_down    (ScintillaObject *sci);
+void				sci_move_selected_lines_up      (ScintillaObject *sci);
+
 #endif
