@@ -1,8 +1,8 @@
 /*
  *      search.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: search.h 5832 2011-06-03 13:40:42Z colombanw $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -30,6 +28,8 @@
 #ifndef GEANY_SEARCH_H
 #define GEANY_SEARCH_H 1
 
+G_BEGIN_DECLS
+
 /* the flags given in the search dialog for "find next", also used by the search bar */
 typedef struct GeanySearchData
 {
@@ -57,9 +57,10 @@ enum GeanyFindSelOptions
 /** Search preferences */
 typedef struct GeanySearchPrefs
 {
-	gboolean	suppress_dialogs;
+	gboolean	always_wrap;			/* don't ask whether to wrap search */
 	gboolean	use_current_word;		/**< Use current word for default search text */
 	gboolean	use_current_file_dir;	/* find in files directory to use on showing dialog */
+	gboolean	hide_find_dialog;		/* hide the find dialog on next or previous */
 	enum GeanyFindSelOptions find_selection_type;
 }
 GeanySearchPrefs;
@@ -99,4 +100,6 @@ gint search_replace_target(struct _ScintillaObject *sci, const gchar *replace_te
 guint search_replace_range(struct _ScintillaObject *sci, struct Sci_TextToFind *ttf,
 		gint flags, const gchar *replace_text);
 
+G_END_DECLS
+
 #endif
