@@ -14,9 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -368,8 +368,9 @@ static gchar *get_doc_folder(const gchar *path)
 	{
 		gsize len = strlen(project_base_path);
 
+		/* remove trailing separator so we can match base path exactly */
 		if (project_base_path[len-1] == G_DIR_SEPARATOR)
-			project_base_path[len-1] = '\0';
+			project_base_path[--len] = '\0';
 
 		/* check whether the dir name matches or uses the project base path */
 		if (utils_filename_has_prefix(tmp_dirname, project_base_path))
@@ -876,11 +877,12 @@ static gboolean openfiles_go_to_selection(GtkTreeSelection *selection, guint key
 }
 
 
-static gboolean taglist_go_to_selection(GtkTreeSelection *selection, guint keyval)
+static gboolean taglist_go_to_selection(GtkTreeSelection *selection, guint keyval, guint state)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gint line = 0;
+	gboolean handled = TRUE;
 
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
@@ -898,13 +900,15 @@ static gboolean taglist_go_to_selection(GtkTreeSelection *selection, guint keyva
 			if (doc != NULL)
 			{
 				navqueue_goto_line(doc, doc, line);
-				if (keyval != GDK_space)
+				if (keyval != GDK_space && ! (state & GDK_CONTROL_MASK))
 					change_focus_to_editor(doc, NULL);
+				else
+					handled = FALSE;
 			}
 		}
 		tm_tag_unref(tag);
 	}
-	return FALSE;
+	return handled;
 }
 
 
@@ -927,7 +931,7 @@ static gboolean sidebar_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 		if (widget == tv.tree_openfiles) /* tag and doc list have separate handlers */
 			openfiles_go_to_selection(selection, event->keyval);
 		else
-			taglist_go_to_selection(selection, event->keyval);
+			taglist_go_to_selection(selection, event->keyval, event->state);
 
 		return TRUE;
 	}
@@ -975,10 +979,12 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 	else if (event->button == 1)
 	{	/* allow reclicking of taglist treeview item */
 		if (widget == tv.tree_openfiles)
+		{
 			openfiles_go_to_selection(selection, 0);
+			handled = TRUE;
+		}
 		else
-			taglist_go_to_selection(selection, 0);
-		handled = TRUE;
+			handled = taglist_go_to_selection(selection, 0, event->state);
 	}
 	else if (event->button == 3)
 	{
