@@ -1,8 +1,8 @@
 /*
  *      stash.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *      MA 02110-1301, USA.
- *
- * $Id: stash.c 5949 2011-09-22 18:04:03Z colombanw $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -68,24 +65,20 @@
  */
 
 /* Implementation Note
- * We use a GArray to hold prefs. It would be more efficient for user code to declare
+ * We dynamically allocate prefs. It would be more efficient for user code to declare
  * a static array of StashPref structs, but we don't do this because:
  *
  * * It would be more ugly (lots of casts and NULLs).
  * * Less type checking.
- * * The API would have to break when adding/changing fields.
+ * * The API & ABI would have to break when adding/changing fields.
  *
  * Usually the prefs code isn't what user code will spend most of its time doing, so this
- * should be efficient enough. But, if desired we could add a stash_group_set_size() function
- * to reduce reallocation (or perhaps use a different container).
- *
- * Note: Maybe using GSlice chunks with an extra 'next' pointer would be more efficient.
+ * should be efficient enough.
  */
 
 
 #include "geany.h"		/* necessary for utils.h, otherwise use gtk/gtk.h */
 #include <stdlib.h>		/* only for atoi() */
-#include <string.h>		/* only for strcmp() */
 #include "support.h"	/* only for _("text") */
 #include "utils.h"		/* only for foreach_*, utils_get_setting_*(). Stash should not depend on Geany. */
 
@@ -100,7 +93,11 @@ struct StashPref
 	gpointer default_value;		/* Default value, e.g. (gpointer)1 */
 	GType widget_type;			/* e.g. GTK_TYPE_TOGGLE_BUTTON */
 	StashWidgetID widget_id;	/* (GtkWidget*) or (gchar*) */
-	gpointer fields;			/* extra fields */
+	union
+	{
+		struct EnumWidget *radio_buttons;
+		const gchar *property_name;
+	} extra;	/* extra fields depending on widget_type */
 };
 
 typedef struct StashPref StashPref;
@@ -108,7 +105,7 @@ typedef struct StashPref StashPref;
 struct StashGroup
 {
 	const gchar *name;			/* group name to use in the keyfile */
-	GArray *entries;			/* array of StashPref */
+	GPtrArray *entries;			/* array of (StashPref*) */
 	gboolean various;		/* mark group for display/edit in stash treeview */
 	gboolean use_defaults;		/* use default values if there's no keyfile entry */
 };
@@ -224,8 +221,9 @@ static void handle_strv_setting(StashGroup *group, StashPref *se,
 static void keyfile_action(SettingAction action, StashGroup *group, GKeyFile *keyfile)
 {
 	StashPref *entry;
+	guint i;
 
-	foreach_array(StashPref, entry, group->entries)
+	foreach_ptr_array(entry, i, group->entries)
 	{
 		/* don't override settings with default values */
 		if (!group->use_defaults && action == SETTING_READ &&
@@ -334,28 +332,51 @@ StashGroup *stash_group_new(const gchar *name)
 	StashGroup *group = g_new0(StashGroup, 1);
 
 	group->name = name;
-	group->entries = g_array_new(FALSE, FALSE, sizeof(StashPref));
+	group->entries = g_ptr_array_new();
 	group->use_defaults = TRUE;
 	return group;
 }
 
 
+/** Frees the memory allocated for setting values in a group.
+ * Useful e.g. to avoid freeing strings individually.
+ * @note This is *not* called by stash_group_free().
+ * @param group . */
+void stash_group_free_settings(StashGroup *group)
+{
+	StashPref *entry;
+	guint i;
+
+	foreach_ptr_array(entry, i, group->entries)
+	{
+		if (entry->setting_type == G_TYPE_STRING)
+			g_free(*(gchararray *) entry->setting);
+		else if (entry->setting_type == G_TYPE_STRV)
+			g_strfreev(*(gchararray **) entry->setting);
+		else
+			continue;
+
+		*(gpointer**) entry->setting = NULL;
+	}
+}
+
+
 /** Frees a group.
  * @param group . */
 void stash_group_free(StashGroup *group)
 {
 	StashPref *entry;
+	guint i;
 
-	foreach_array(StashPref, entry, group->entries)
+	foreach_ptr_array(entry, i, group->entries)
 	{
 		if (entry->widget_type == GTK_TYPE_RADIO_BUTTON)
-			g_free(entry->fields);
-		else if (entry->widget_type == G_TYPE_PARAM)
-			continue;
-		else
-			g_assert(entry->fields == NULL);	/* to prevent memory leaks, must handle fields above */
+		{
+			g_free(entry->extra.radio_buttons);
+		}
+		g_slice_free(StashPref, entry);
 	}
-	g_array_free(group->entries, TRUE);
+	g_ptr_array_free(group->entries, TRUE);
 	g_free(group);
 }
 
@@ -381,8 +402,10 @@ static StashPref *
 add_pref(StashGroup *group, GType type, gpointer setting,
 		const gchar *key_name, gpointer default_value)
 {
-	StashPref entry = {type, setting, key_name, default_value, G_TYPE_NONE, NULL, NULL};
-	GArray *array = group->entries;
+	StashPref init = {type, setting, key_name, default_value, G_TYPE_NONE, NULL, {NULL}};
+	StashPref *entry = g_slice_new(StashPref);
+
+	*entry = init;
 
 	/* init any pointer settings to NULL so they can be freed later */
 	if (type == G_TYPE_STRING ||
@@ -390,9 +413,8 @@ add_pref(StashGroup *group, GType type, gpointer setting,
 		if (group->use_defaults)
 			*(gpointer**)setting = NULL;
 
-	g_array_append_val(array, entry);
-
-	return &g_array_index(array, StashPref, array->len - 1);
+	g_ptr_array_add(group->entries, entry);
+	return entry;
 }
 
 
@@ -533,22 +555,25 @@ lookup_widget(GtkWidget *widget, const gchar *widget_name)
 {
 	GtkWidget *parent, *found_widget;
 
-	for (;;)
-		{
-			if (GTK_IS_MENU (widget))
-				parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
-			else
-				parent = widget->parent;
-			if (!parent)
-				parent = (GtkWidget*) g_object_get_data (G_OBJECT (widget), "GladeParentKey");
-			if (parent == NULL)
-				break;
-			widget = parent;
-		}
+	g_return_val_if_fail(widget != NULL, NULL);
+	g_return_val_if_fail(widget_name != NULL, NULL);
 
-	found_widget = (GtkWidget*) g_object_get_data (G_OBJECT (widget), widget_name);
-	if (!found_widget)
-		g_warning ("Widget not found: %s", widget_name);
+	for (;;)
+	{
+		if (GTK_IS_MENU(widget))
+			parent = gtk_menu_get_attach_widget(GTK_MENU(widget));
+		else
+			parent = gtk_widget_get_parent(widget);
+		if (parent == NULL)
+			parent = (GtkWidget*) g_object_get_data(G_OBJECT(widget), "GladeParentKey");
+		if (parent == NULL)
+			break;
+		widget = parent;
+	}
+
+	found_widget = (GtkWidget*) g_object_get_data(G_OBJECT(widget), widget_name);
+	if (G_UNLIKELY(found_widget == NULL))
+		g_warning("Widget not found: %s", widget_name);
 	return found_widget;
 }
 
@@ -589,11 +614,10 @@ static void handle_radio_button(GtkWidget *widget, gint enum_id, gboolean *setti
 }
 
 
-static void handle_radio_buttons(GtkWidget *owner, EnumWidget *fields,
-		gboolean *setting,
+static void handle_radio_buttons(GtkWidget *owner, StashPref *entry,
 		PrefAction action)
 {
-	EnumWidget *field = fields;
+	EnumWidget *field = entry->extra.radio_buttons;
 	gsize count = 0;
 	GtkWidget *widget = NULL;
 
@@ -605,7 +629,7 @@ static void handle_radio_buttons(GtkWidget *owner, EnumWidget *fields,
 			continue;
 
 		count++;
-		handle_radio_button(widget, field->enum_id, setting, action);
+		handle_radio_button(widget, field->enum_id, entry->setting, action);
 		field++;
 		if (!field->widget_id)
 			break;
@@ -619,7 +643,7 @@ static void handle_widget_property(GtkWidget *widget, StashPref *entry,
 		PrefAction action)
 {
 	GObject *object = G_OBJECT(widget);
-	const gchar *name = entry->fields;
+	const gchar *name = entry->extra.property_name;
 
 	switch (action)
 	{
@@ -640,8 +664,9 @@ static void handle_widget_property(GtkWidget *widget, StashPref *entry,
 static void pref_action(PrefAction action, StashGroup *group, GtkWidget *owner)
 {
 	StashPref *entry;
+	guint i;
 
-	foreach_array(StashPref, entry, group->entries)
+	foreach_ptr_array(entry, i, group->entries)
 	{
 		GtkWidget *widget;
 
@@ -652,7 +677,7 @@ static void pref_action(PrefAction action, StashGroup *group, GtkWidget *owner)
 		/* radio buttons have several widgets */
 		if (entry->widget_type == GTK_TYPE_RADIO_BUTTON)
 		{
-			handle_radio_buttons(owner, entry->fields, entry->setting, action);
+			handle_radio_buttons(owner, entry, action);
 			continue;
 		}
 
@@ -774,7 +799,7 @@ void stash_group_add_radio_buttons(StashGroup *group, gint *setting,
 	va_end(args);
 
 	array = g_new0(EnumWidget, count + 1);
-	entry->fields = array;
+	entry->extra.radio_buttons = array;
 
 	va_start(args, enum_id);
 	foreach_c_array(item, array, count)
@@ -886,7 +911,7 @@ void stash_group_add_widget_property(StashGroup *group, gpointer setting,
 		type = object_get_property_type(G_OBJECT(widget_id), property_name);
 
 	add_widget_pref(group, type, setting, key_name, default_value,
-		G_TYPE_PARAM, widget_id)->fields = (gchar*)property_name;
+		G_TYPE_PARAM, widget_id)->extra.property_name = property_name;
 }
 
 
@@ -900,10 +925,13 @@ enum
 
 struct StashTreeValue
 {
-	GType setting_type;
-	gpointer setting;
-	const gchar *key_name;
 	const gchar *group_name;
+	StashPref *pref;
+	union
+	{
+		gchararray tree_string;
+		gint tree_int;
+	} data;
 };
 
 typedef struct StashTreeValue StashTreeValue;
@@ -914,29 +942,31 @@ static void stash_tree_renderer_set_data(GtkCellLayout *cell_layout, GtkCellRend
 {
 	GType cell_type = GPOINTER_TO_SIZE(user_data);
 	StashTreeValue *value;
+	StashPref *pref;
 	gboolean matches_type;
 
 	gtk_tree_model_get(model, iter, STASH_TREE_VALUE, &value, -1);
-	matches_type = value->setting_type == cell_type;
+	pref = value->pref;
+	matches_type = pref->setting_type == cell_type;
 	g_object_set(cell, "visible", matches_type, "sensitive", matches_type,
 		cell_type == G_TYPE_BOOLEAN ? "activatable" : "editable", matches_type, NULL);
 
 	if (matches_type)
 	{
-		switch (value->setting_type)
+		switch (pref->setting_type)
 		{
 			case G_TYPE_BOOLEAN:
-				g_object_set(cell, "active", GPOINTER_TO_INT(value->setting), NULL);
+				g_object_set(cell, "active", value->data.tree_int, NULL);
 				break;
 			case G_TYPE_INT:
 			{
-				gchar *text = g_strdup_printf("%d", GPOINTER_TO_INT(value->setting));
+				gchar *text = g_strdup_printf("%d", value->data.tree_int);
 				g_object_set(cell, "text", text, NULL);
 				g_free(text);
 				break;
 			}
 			case G_TYPE_STRING:
-				g_object_set(cell, "text", (gchararray) value->setting, NULL);
+				g_object_set(cell, "text", value->data.tree_string, NULL);
 				break;
 		}
 	}
@@ -948,22 +978,23 @@ static void stash_tree_renderer_edited(gchar *path_str, gchar *new_text, GtkTree
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	StashTreeValue *value;
+	StashPref *pref;
 
 	path = gtk_tree_path_new_from_string(path_str);
 	gtk_tree_model_get_iter(model, &iter, path);
 	gtk_tree_model_get(model, &iter, STASH_TREE_VALUE, &value, -1);
+	pref = value->pref;
 
-	switch (value->setting_type)
+	switch (pref->setting_type)
 	{
 		case G_TYPE_BOOLEAN:
-			value->setting = GINT_TO_POINTER(!GPOINTER_TO_INT(value->setting));
+			value->data.tree_int = !value->data.tree_int;
 			break;
 		case G_TYPE_INT:
-			value->setting = GINT_TO_POINTER(atoi(new_text));
+			value->data.tree_int = atoi(new_text);
 			break;
 		case G_TYPE_STRING:
-			g_free(value->setting);
-			value->setting = g_strdup(new_text);
+			SETPTR(value->data.tree_string, g_strdup(new_text));
 			break;
 	}
 
@@ -992,8 +1023,8 @@ static gboolean stash_tree_discard_value(GtkTreeModel *model, GtkTreePath *path,
 	StashTreeValue *value;
 
 	gtk_tree_model_get(model, iter, STASH_TREE_VALUE, &value, -1);
-	if (value->setting_type == G_TYPE_STRING)
-		g_free(value->setting);
+	if (value->pref->setting_type == G_TYPE_STRING)
+		g_free(value->data.tree_string);
 	g_free(value);
 
 	return FALSE;
@@ -1007,46 +1038,41 @@ static void stash_tree_destroy_cb(GtkWidget *widget, gpointer user_data)
 }
 
 
-typedef void (*stash_foreach_pref_func)(StashGroup *group, StashPref *entry, gpointer container,
-	PrefAction action);
-
-static void stash_foreach_various_pref(GPtrArray *group_array, stash_foreach_pref_func pref_func,
-	gpointer container, PrefAction action)
-{
-	StashGroup *group;
-	guint i;
-	StashPref *entry;
-
-	foreach_ptr_array(group, i, group_array)
-	{
-		if (group->various)
-		{
-			foreach_array(StashPref, entry, group->entries)
-				pref_func(group, entry, container, action);
-		}
-	}
-}
-
-
 static void stash_tree_append_pref(StashGroup *group, StashPref *entry, GtkListStore *store,
 	PrefAction action)
 {
 	GtkTreeIter iter;
 	StashTreeValue *value;
 
-	value = g_new(StashTreeValue, 1);
+	value = g_new0(StashTreeValue, 1);
 
-	value->setting_type = entry->setting_type;
-	value->setting = NULL;
-	value->key_name = entry->key_name;
 	value->group_name = group->name;
+	value->pref = entry;
 
 	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, STASH_TREE_NAME, value->key_name,
+	gtk_list_store_set(store, &iter, STASH_TREE_NAME, entry->key_name,
 		STASH_TREE_VALUE, value, -1);
 }
 
 
+static void stash_tree_append_prefs(GPtrArray *group_array,
+	GtkListStore *store, PrefAction action)
+{
+	StashGroup *group;
+	guint i, j;
+	StashPref *entry;
+
+	foreach_ptr_array(group, i, group_array)
+	{
+		if (group->various)
+		{
+			foreach_ptr_array(entry, j, group->entries)
+				stash_tree_append_pref(group, entry, store, action);
+		}
+	}
+}
+
+
 /* Setups a simple editor for stash preferences based on the widget arguments.
  * group_array - Array of groups which's settings will be edited.
  * tree - GtkTreeView in which to edit the preferences. Must be empty. */
@@ -1056,11 +1082,10 @@ void stash_tree_setup(GPtrArray *group_array, GtkTreeView *tree)
 	GtkTreeModel *model;
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
-	GtkObject *adjustment;
+	GtkAdjustment *adjustment;
 
 	store = gtk_list_store_new(STASH_TREE_COUNT, G_TYPE_STRING, G_TYPE_POINTER);
-	stash_foreach_various_pref(group_array,
-		(stash_foreach_pref_func) stash_tree_append_pref, store, PREF_DISPLAY);
+	stash_tree_append_prefs(group_array, store, PREF_DISPLAY);
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), STASH_TREE_NAME,
 		GTK_SORT_ASCENDING);
 
@@ -1094,7 +1119,7 @@ void stash_tree_setup(GPtrArray *group_array, GtkTreeView *tree)
 		stash_tree_renderer_set_data, GSIZE_TO_POINTER(G_TYPE_STRING), NULL);
 	/* integer renderer */
 	cell = gtk_cell_renderer_spin_new();
-	adjustment = gtk_adjustment_new(0, G_MININT, G_MAXINT, 1, 10, 0);
+	adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(0, G_MININT, G_MAXINT, 1, 10, 0));
 	g_object_set(cell, "adjustment", adjustment, NULL);
 	g_signal_connect(cell, "edited", G_CALLBACK(stash_tree_string_edited), model);
 	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(column), cell, FALSE);
@@ -1108,17 +1133,14 @@ static void stash_tree_display_pref(StashTreeValue *value, StashPref *entry)
 	switch (entry->setting_type)
 	{
 		case G_TYPE_BOOLEAN:
-			value->setting = GINT_TO_POINTER(*(gboolean *) entry->setting);
+			value->data.tree_int = *(gboolean *) entry->setting;
 			break;
 		case G_TYPE_INT:
-			value->setting = GINT_TO_POINTER(*(gint *) entry->setting);
+			value->data.tree_int = *(gint *) entry->setting;
 			break;
 		case G_TYPE_STRING:
-		{
-			g_free(value->setting);
-			value->setting = g_strdup(*(gchararray *) entry->setting);
+			SETPTR(value->data.tree_string, g_strdup(*(gchararray *) entry->setting));
 			break;
-		}
 		default:
 			g_warning("Unhandled type for %s::%s in %s()!", value->group_name,
 				entry->key_name, G_STRFUNC);
@@ -1128,33 +1150,28 @@ static void stash_tree_display_pref(StashTreeValue *value, StashPref *entry)
 
 static void stash_tree_update_pref(StashTreeValue *value, StashPref *entry)
 {
-	gpointer *setting = value->setting;
-
 	switch (entry->setting_type)
 	{
 		case G_TYPE_BOOLEAN:
-			*(gboolean *) entry->setting = GPOINTER_TO_INT(setting);
+			*(gboolean *) entry->setting = value->data.tree_int;
 			break;
 		case G_TYPE_INT:
-			*(gint *) entry->setting = GPOINTER_TO_INT(setting);
+			*(gint *) entry->setting = value->data.tree_int;
 			break;
 		case G_TYPE_STRING:
 		{
 			gchararray *text = entry->setting;
-			g_free(*text);
-			*text = g_strdup((gchararray) setting);
+			SETPTR(*text, g_strdup(value->data.tree_string));
 			break;
 		}
 		default:
 			g_warning("Unhandled type for %s::%s in %s()!", value->group_name,
-				value->key_name, G_STRFUNC);
+				entry->key_name, G_STRFUNC);
 	}
 }
 
-/* These functions can handle about 200 settings on a 1GHz x86 CPU in ~0.06 seconds.
- * For 250+ settings, you'd better write something more efficient. */
-static void stash_tree_handle_pref(StashGroup *group, StashPref *entry, GtkTreeModel *model,
-	PrefAction action)
+
+static void stash_tree_action(GtkTreeModel *model, PrefAction action)
 {
 	GtkTreeIter iter;
 	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
@@ -1164,36 +1181,27 @@ static void stash_tree_handle_pref(StashGroup *group, StashPref *entry, GtkTreeM
 	{
 		gtk_tree_model_get(model, &iter, STASH_TREE_VALUE, &value, -1);
 
-		if (strcmp(group->name, value->group_name) == 0 &&
-			strcmp(entry->key_name, value->key_name) == 0)
+		switch (action)
 		{
-			switch (action)
-			{
-				case PREF_DISPLAY:
-					stash_tree_display_pref(value, entry);
-					break;
-				case PREF_UPDATE:
-					stash_tree_update_pref(value, entry);
-					break;
-			}
-
-			break;
+			case PREF_DISPLAY:
+				stash_tree_display_pref(value, value->pref);
+				break;
+			case PREF_UPDATE:
+				stash_tree_update_pref(value, value->pref);
+				break;
 		}
-
 		valid = gtk_tree_model_iter_next(model, &iter);
 	}
 }
 
 
-void stash_tree_display(GPtrArray *group_array, GtkTreeView *tree)
+void stash_tree_display(GtkTreeView *tree)
 {
-	stash_foreach_various_pref(group_array, (stash_foreach_pref_func) stash_tree_handle_pref,
-		gtk_tree_view_get_model(tree), PREF_DISPLAY);
+	stash_tree_action(gtk_tree_view_get_model(tree), PREF_DISPLAY);
 }
 
 
-void stash_tree_update(GPtrArray *group_array, GtkTreeView *tree)
+void stash_tree_update(GtkTreeView *tree)
 {
-	stash_foreach_various_pref(group_array, (stash_foreach_pref_func) stash_tree_handle_pref,
-		gtk_tree_view_get_model(tree), PREF_UPDATE);
+	stash_tree_action(gtk_tree_view_get_model(tree), PREF_UPDATE);
 }
