@@ -14,9 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
@@ -61,4 +61,6 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition);
 
 gint symbols_get_current_function(GeanyDocument *doc, const gchar **tagname);
 
+gint symbols_get_current_scope(GeanyDocument *doc, const gchar **tagname);
+
 #endif
