@@ -14,9 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -72,7 +72,7 @@ struct _VteTerminal
 	GtkAdjustment *adjustment;
 };
 
-#define VTE_TERMINAL(obj) (GTK_CHECK_CAST((obj), VTE_TYPE_TERMINAL, VteTerminal))
+#define VTE_TERMINAL(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), VTE_TYPE_TERMINAL, VteTerminal))
 #define VTE_TYPE_TERMINAL (vf->vte_terminal_get_type())
 
 typedef enum {
@@ -93,7 +93,7 @@ struct VteFunctions
 	void (*vte_terminal_set_word_chars) (VteTerminal *terminal, const char *spec);
 	void (*vte_terminal_set_mouse_autohide) (VteTerminal *terminal, gboolean setting);
 	void (*vte_terminal_reset) (VteTerminal *terminal, gboolean full, gboolean clear_history);
-	GtkType (*vte_terminal_get_type) (void);
+	GType (*vte_terminal_get_type) (void);
 	void (*vte_terminal_set_scroll_on_output) (VteTerminal *terminal, gboolean scroll);
 	void (*vte_terminal_set_scroll_on_keystroke) (VteTerminal *terminal, gboolean scroll);
 	void (*vte_terminal_set_font_from_string) (VteTerminal *terminal, const char *name);
@@ -112,6 +112,7 @@ struct VteFunctions
 	void (*vte_terminal_set_cursor_blinks) (VteTerminal *terminal, gboolean blink);
 	void (*vte_terminal_select_all) (VteTerminal *terminal);
 	void (*vte_terminal_set_audible_bell) (VteTerminal *terminal, gboolean is_audible);
+	void (*vte_terminal_set_background_image_file) (VteTerminal *terminal, const char *path);
 };
 
 
@@ -121,7 +122,7 @@ static void vte_restart(GtkWidget *widget);
 static gboolean vte_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 static gboolean vte_keyrelease_cb(GtkWidget *widget, GdkEventKey *event, gpointer data);
 static gboolean vte_keypress_cb(GtkWidget *widget, GdkEventKey *event, gpointer data);
-static void vte_register_symbols(GModule *module);
+static gboolean vte_register_symbols(GModule *module);
 static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data);
 static GtkWidget *vte_create_popup_menu(void);
 static void vte_commit_cb(VteTerminal *vte, gchar *arg1, guint arg2, gpointer user_data);
@@ -216,9 +217,18 @@ void vte_init(void)
 	}
 	else
 	{
-		vte_info.have_vte = TRUE;
 		vf = g_new0(struct VteFunctions, 1);
-		vte_register_symbols(module);
+		if (vte_register_symbols(module))
+			vte_info.have_vte = TRUE;
+		else
+		{
+			vte_info.have_vte = FALSE;
+			g_free(vf);
+			/* FIXME: is closing the module safe? see vte_close() and test on FreeBSD */
+			/*g_module_close(module);*/
+			module = NULL;
+			return;
+		}
 	}
 
 	create_vte();
@@ -294,6 +304,7 @@ void vte_close(void)
 	g_object_unref(vc->menu);
 	g_free(vc->emulation);
 	g_free(vc->shell);
+	g_free(vc->image);
 	g_free(vc->font);
 	g_free(vc->colour_back);
 	g_free(vc->colour_fore);
@@ -390,6 +401,10 @@ static gboolean vte_button_pressed(GtkWidget *widget, GdkEventButton *event, gpo
 		gtk_widget_grab_focus(vc->vte);
 		gtk_menu_popup(GTK_MENU(vc->menu), NULL, NULL, NULL, NULL, event->button, event->time);
 	}
+	else if (event->button == 2)
+	{
+		gtk_widget_grab_focus(widget);
+	}
 	return FALSE;
 }
 
@@ -406,35 +421,52 @@ static void vte_set_cursor_blink_mode(void)
 }
 
 
-static void vte_register_symbols(GModule *mod)
+static gboolean vte_register_symbols(GModule *mod)
 {
-	g_module_symbol(mod, "vte_terminal_new", (void*)&vf->vte_terminal_new);
-	g_module_symbol(mod, "vte_terminal_set_size", (void*)&vf->vte_terminal_set_size);
-	g_module_symbol(mod, "vte_terminal_fork_command", (void*)&vf->vte_terminal_fork_command);
-	g_module_symbol(mod, "vte_terminal_set_word_chars", (void*)&vf->vte_terminal_set_word_chars);
-	g_module_symbol(mod, "vte_terminal_set_mouse_autohide", (void*)&vf->vte_terminal_set_mouse_autohide);
-	g_module_symbol(mod, "vte_terminal_reset", (void*)&vf->vte_terminal_reset);
-	g_module_symbol(mod, "vte_terminal_get_type", (void*)&vf->vte_terminal_get_type);
-	g_module_symbol(mod, "vte_terminal_set_scroll_on_output", (void*)&vf->vte_terminal_set_scroll_on_output);
-	g_module_symbol(mod, "vte_terminal_set_scroll_on_keystroke", (void*)&vf->vte_terminal_set_scroll_on_keystroke);
-	g_module_symbol(mod, "vte_terminal_set_font_from_string", (void*)&vf->vte_terminal_set_font_from_string);
-	g_module_symbol(mod, "vte_terminal_set_scrollback_lines", (void*)&vf->vte_terminal_set_scrollback_lines);
-	g_module_symbol(mod, "vte_terminal_get_has_selection", (void*)&vf->vte_terminal_get_has_selection);
-	g_module_symbol(mod, "vte_terminal_copy_clipboard", (void*)&vf->vte_terminal_copy_clipboard);
-	g_module_symbol(mod, "vte_terminal_paste_clipboard", (void*)&vf->vte_terminal_paste_clipboard);
-	g_module_symbol(mod, "vte_terminal_set_emulation", (void*)&vf->vte_terminal_set_emulation);
-	g_module_symbol(mod, "vte_terminal_set_color_foreground", (void*)&vf->vte_terminal_set_color_foreground);
-	g_module_symbol(mod, "vte_terminal_set_color_bold", (void*)&vf->vte_terminal_set_color_bold);
-	g_module_symbol(mod, "vte_terminal_set_color_background", (void*)&vf->vte_terminal_set_color_background);
-	g_module_symbol(mod, "vte_terminal_feed_child", (void*)&vf->vte_terminal_feed_child);
-	g_module_symbol(mod, "vte_terminal_im_append_menuitems", (void*)&vf->vte_terminal_im_append_menuitems);
-	g_module_symbol(mod, "vte_terminal_set_cursor_blink_mode", (void*)&vf->vte_terminal_set_cursor_blink_mode);
-	if (vf->vte_terminal_set_cursor_blink_mode == NULL)
+	#define BIND_SYMBOL(field) \
+		g_module_symbol(mod, #field, (void*)&vf->field)
+	#define BIND_REQUIRED_SYMBOL(field) \
+		G_STMT_START { \
+			if (! BIND_SYMBOL(field)) \
+			{ \
+				g_critical(_("invalid VTE library \"%s\": missing symbol \"%s\""), \
+						g_module_name(mod), #field); \
+				return FALSE; \
+			} \
+		} G_STMT_END
+
+	BIND_REQUIRED_SYMBOL(vte_terminal_new);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_size);
+	BIND_REQUIRED_SYMBOL(vte_terminal_fork_command);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_word_chars);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_mouse_autohide);
+	BIND_REQUIRED_SYMBOL(vte_terminal_reset);
+	BIND_REQUIRED_SYMBOL(vte_terminal_get_type);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_scroll_on_output);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_scroll_on_keystroke);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_font_from_string);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_scrollback_lines);
+	BIND_REQUIRED_SYMBOL(vte_terminal_get_has_selection);
+	BIND_REQUIRED_SYMBOL(vte_terminal_copy_clipboard);
+	BIND_REQUIRED_SYMBOL(vte_terminal_paste_clipboard);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_emulation);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_color_foreground);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_color_bold);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_color_background);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_background_image_file);
+	BIND_REQUIRED_SYMBOL(vte_terminal_feed_child);
+	BIND_REQUIRED_SYMBOL(vte_terminal_im_append_menuitems);
+	if (! BIND_SYMBOL(vte_terminal_set_cursor_blink_mode))
 		/* vte_terminal_set_cursor_blink_mode() is only available since 0.17.1, so if we don't find
 		 * this symbol, we are probably on an older version and use the old API instead */
-		g_module_symbol(mod, "vte_terminal_set_cursor_blinks", (void*)&vf->vte_terminal_set_cursor_blinks);
-	g_module_symbol(mod, "vte_terminal_select_all", (void*)&vf->vte_terminal_select_all);
-	g_module_symbol(mod, "vte_terminal_set_audible_bell", (void*)&vf->vte_terminal_set_audible_bell);
+		BIND_REQUIRED_SYMBOL(vte_terminal_set_cursor_blinks);
+	BIND_REQUIRED_SYMBOL(vte_terminal_select_all);
+	BIND_REQUIRED_SYMBOL(vte_terminal_set_audible_bell);
+
+	#undef BIND_REQUIRED_SYMBOL
+	#undef BIND_SYMBOL
+
+	return TRUE;
 }
 
 
@@ -451,6 +483,7 @@ void vte_apply_user_settings(void)
 	vf->vte_terminal_set_color_foreground(VTE_TERMINAL(vc->vte), vc->colour_fore);
 	vf->vte_terminal_set_color_bold(VTE_TERMINAL(vc->vte), vc->colour_fore);
 	vf->vte_terminal_set_color_background(VTE_TERMINAL(vc->vte), vc->colour_back);
+	vf->vte_terminal_set_background_image_file(VTE_TERMINAL(vc->vte), vc->image);
 	vf->vte_terminal_set_audible_bell(VTE_TERMINAL(vc->vte), prefs.beep_on_errors);
 	vte_set_cursor_blink_mode();
 
@@ -684,9 +717,10 @@ static void vte_drag_data_received(GtkWidget *widget, GdkDragContext *drag_conte
 {
 	if (info == TARGET_TEXT_PLAIN)
 	{
-		if (data->format == 8 && data->length > 0)
+		if (gtk_selection_data_get_format(data) == 8 && gtk_selection_data_get_length(data) > 0)
 			vf->vte_terminal_feed_child(VTE_TERMINAL(widget),
-				(const gchar*) data->data, data->length);
+				(const gchar*) gtk_selection_data_get_data(data),
+				gtk_selection_data_get_length(data));
 	}
 	else
 	{
@@ -741,12 +775,18 @@ void vte_append_preferences_tab(void)
 		GtkWidget *frame_term, *button_shell, *entry_shell;
 		GtkWidget *check_run_in_vte, *check_skip_script;
 		GtkWidget *font_button, *fg_color_button, *bg_color_button;
+		GtkWidget *entry_image, *button_image;
 
 		button_shell = GTK_WIDGET(ui_lookup_widget(ui_widgets.prefs_dialog, "button_term_shell"));
 		entry_shell = GTK_WIDGET(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_shell"));
 		ui_setup_open_button_callback(button_shell, NULL,
 			GTK_FILE_CHOOSER_ACTION_OPEN, GTK_ENTRY(entry_shell));
 
+		button_image = GTK_WIDGET(ui_lookup_widget(ui_widgets.prefs_dialog, "button_term_image"));
+		entry_image = GTK_WIDGET(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_image"));
+		ui_setup_open_button_callback(button_image, NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN, GTK_ENTRY(entry_image));
+
 		check_skip_script = GTK_WIDGET(ui_lookup_widget(ui_widgets.prefs_dialog, "check_skip_script"));
 		gtk_widget_set_sensitive(check_skip_script, vc->run_in_vte);
 
